#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import numpy as np
from copy import deepcopy
from antaress.ANTARESS_general.utils import stop

def gen_plot_default(plot_options,key_plot,plot_dic,gen_dic):
    r"""**Default plot settings.**
    
    Set default values for the following fields. 
    Manual values can be given to all fields through `ANTARESS_plot_settings.py`, depending on their relevance for the chosen plot function. 
    
     - `fig_size = (width,height)` : figure size.
     - `margins = [x_left,y_low,x_width,y_width]` : margins of plot in the figure. 
     - `wspace = width` : horizontal separation between subplots.      
       `hspace = height` : vertical separation between subplots. 
     - `title = bool` : show plot title.
     - `font_size = float` : size for figure font.     
     - `font_size_txt = float` : size for text font within plot.
     
    Args:
        plot_options (dic) : dictionary for all generic plot settings
        key_plot (str) : identifier of current plot 
    
    Returns:
        plot_options (dic) : input, initialized with default settings according to the plot identifier
    
    """
    plot_options[key_plot]={}
    
    #Figure size
    plot_options[key_plot]['fig_size'] = (10,6)
    
    #Margins
    plot_options[key_plot]['margins']=[0.15,0.15,0.95,0.7] 
    
    #Separation spaces in subplots 
    plot_options[key_plot]['wspace'] = 0.06
    plot_options[key_plot]['hspace'] = 0.06
    
    #Plot title
    plot_options[key_plot]['title']= False        
    
    #Font size
    plot_options[key_plot]['font_size']=14
    plot_options[key_plot]['font_size_txt'] = deepcopy(plot_options[key_plot]['font_size'])

    #Linewidth
    plot_options[key_plot]['lw_plot']=0.5
    
    #Linestyle
    plot_options[key_plot]['ls_plot']='-'
    
    #Axis thickness
    plot_options[key_plot]['axis_thick']=1  
    
    #Marker
    plot_options[key_plot]['marker'] = 'o'
    
    #Symbol size
    plot_options[key_plot]['markersize']=2.5

    #Hide axis
    plot_options[key_plot]['hide_axis'] = False
    
    #Rasterize datapoints
    plot_options[key_plot]['rasterized'] = True

    #Transparent background
    plot_options[key_plot]['transparent'] = False
    
    #Curve style
    plot_options[key_plot]['drawstyle']=None
 
    #Plot abscissa range or errors
    plot_options[key_plot]['plot_xerr']=False  

    #Transparency of symbols (0 = void)
    plot_options[key_plot]['alpha_symb']=1. 
  
    #Transparency of error bars (0 = void)
    plot_options[key_plot]['alpha_err']=0.5

    #Spectral resampling
    plot_options[key_plot]['resample'] = None
    
    #List of stellar lines to plot in spectral mode
    #    - default to stellar CCF mask
    plot_options[key_plot]['st_lines_wav'] = []

    #List of planet lines to plot in spectral mode
    #    - default to planet CCF mask
    plot_options[key_plot]['pl_lines_wav'] = []

    #Make GIF from plot series.
    plot_options[key_plot]['GIF_generation'] = False
    
    #FPS for gif
    plot_options[key_plot]['fps'] = 5

    #--------------------------------------

    #Instruments to plot
    plot_options[key_plot]['inst_to_plot']=list(plot_dic['visits_to_plot'].keys())
    
    #Visits to plot
    #    - add '_bin' to the name of a visit to plot properties derived from intrinsic profiles binned within a visit
    #      use 'binned' as visit name to plot properties derived from intrinsic profiles binned over several visits
    plot_options[key_plot]['visits_to_plot']=plot_dic['visits_to_plot']
    
    #Indexes of exposures to be plotted 
    plot_options[key_plot]['iexp_plot']={}

    #Indexes of exposures to be removed from the plot
    plot_options[key_plot]['idx_noplot'] = {}

    #Do not plot original data
    plot_options[key_plot]['no_orig']= False

    #Plot data
    plot_options[key_plot]['plot_data'] = True

    #Plot data errors
    plot_options[key_plot]['plot_err']=True 

    #Plot models 
    plot_options[key_plot]['plot_model'] = True

    #Indexes of orders to be plotted         
    #    - leave empty to plot all orders
    plot_options[key_plot]['orders_to_plot']=[]    
    
    #Colors
    plot_options[key_plot]['color_dic']={}  
    plot_options[key_plot]['color_dic_sec']={}
    plot_options[key_plot]['color_dic_bin']={}  
    plot_options[key_plot]['color_dic_bin_sec']={}
    
    #Scaling factor 
    #    - in power of ten, ie flux are multiplied by 10**sc_fact10)
    plot_options[key_plot]['sc_fact10'] = 0.
    
    #Spectral variable
    #    - wavelength in A
    plot_options[key_plot]['sp_var'] = 'wav'
    
    #Bornes du plot  
    plot_options[key_plot]['x_range']=None
    plot_options[key_plot]['y_range']=None 

    #Overplot excluded planetary ranges
    plot_options[key_plot]['plot_plexc']=False

    #Overplot intrinsic stellar line ranges
    plot_options[key_plot]['plot_stexc']=False
    
    #Reference planet for each visit
    plot_options[key_plot]['pl_ref']={}
    for inst in plot_options[key_plot]['visits_to_plot']:
        plot_options[key_plot]['pl_ref'][inst]={'binned':gen_dic['studied_pl'][0]}
        for vis in plot_options[key_plot]['visits_to_plot'][inst]:
            plot_options[key_plot]['pl_ref'][inst][vis]=gen_dic['studied_pl'][0]

    #Shade range not used for fitting
    plot_options[key_plot]['shade_unfit']=False
    
    #Shade selected ranges
    plot_options[key_plot]['shade_ranges']={}
    
    #Plot continuum pixels common to all exposures
    plot_options[key_plot]['plot_cont']=False

    #Shade continuum range requested as input
    plot_options[key_plot]['shade_cont']=False     

    #Plot continuum level
    plot_options[key_plot]['plot_cont_lev']=False
    
    #Plot continuum pixels specific to each exposure
    plot_options[key_plot]['plot_cont_exp']=False   
    
    #Plot residuals from continuum alone
    plot_options[key_plot]['cont_only'] = False

    #Plot fitted pixels
    plot_options[key_plot]['plot_fitpix']=False

    #Plot fitted line profile
    plot_options[key_plot]['plot_line_model']=False

    #Plot model line profile
    plot_options[key_plot]['plot_line_model_HR']=False

    #Plot individual model components for line profile
    plot_options[key_plot]['plot_line_model_compo']=False
    
    #Print fit properties on plot
    plot_options[key_plot]['plot_prop']=True  

    #Plot fitted line centroid
    plot_options[key_plot]['plot_line_fit_rv']=False

    #Plot bissector
    plot_options[key_plot]['plot_biss']=False
            
    #Print dispersions
    plot_options[key_plot]['print_disp'] = []  
    
    #Plot HITRAN telluric lines for requested molecules
    plot_options[key_plot]['plot_tell_HITRANS']=[]
    plot_options[key_plot]['telldepth_min'] = 0.
    
    #Plot master used as reference for flux balance correction
    plot_options[key_plot]['plot_mast'] = False        
    
    #Plot spectra at two chosen steps of the correction process
    #    - set to None, or chose amongst:
    # + 'raw' : before any correction
    # + 'all' : after all requested corrections
    # + 'tell' : after telluric correction 
    # + 'count' : after flux-to-count scaling
    # + 'fbal_glob' : after global flux balance correction 
    # + 'fbal_ord' : after order/orde flux balance correction  
    # + 'cosm' : after cosmics correction  
    # + 'permpeak' : after persistent peak correction 
    # + 'wig' : after wiggle correction 
    plot_options[key_plot]['plot_pre']='all'
    plot_options[key_plot]['plot_post']=None        
 
    #Plot all exposures on the same plot
    plot_options[key_plot]['multi_exp']= False

    #Plot all order on the same plot
    plot_options[key_plot]['multi_ord']= False    

    #Normalize spectra to integrated flux unity
    #    - to allow for comparison
    plot_options[key_plot]['norm_prof'] = False  

    #Absorption signal type
    plot_options[key_plot]['pl_atm_sign']='Absorption'  

    #Histograms bin number
    plot_options[key_plot]['bins_par'] = 40
    
    #Exposures used in master out
    plot_options[key_plot]['iexp_mast_list']={}

    #--------------------------------------   

    #Aligned profiles
    plot_options[key_plot]['aligned']=False

    #Measured values
    plot_options[key_plot]['print_mes']=False
    
    #Aligned profiles
    plot_options[key_plot]['ref_level']=False

    #Plot reference velocity
    plot_options[key_plot]['plot_refvel']=True  

    #Fit type
    plot_options[key_plot]['fit_type']='indiv'

    #Plot fitted exposures only
    plot_options[key_plot]['fitted_exp'] = False

    #Planet-occulted line model from best-fit ('fit') or reconstruction ('rec')
    plot_options[key_plot]['line_model']='fit'
    
    #Ranges for continuum definition in plots
    plot_options[key_plot]['cont_range']={}

    #--------------------------------------  
    if (key_plot in ['glob_mast','all_DI_data','all_intr_data','all_atm_data']):
        
        #Plot spectra used for measured master calculation
        plot_options[key_plot]['plot_input']=True
        
    #--------------------------------------              
    if (key_plot in ['Fbal_corr','input_LC','plocc_ranges','prop_DI_mcmc_PDFs','prop_Intr_mcmc_PDFs']):

        #Plot exposure indexes
        plot_options[key_plot]['plot_expid'] = True
        
    #--------------------------------------   
    #2D map settings     
    if ('map_' in key_plot) or ('bin' in key_plot) or ('prop_' in key_plot) or (key_plot in ['occulted_regions']):

        #Bin dimension
        #    - 'phase' (see details and routine)
        #    - coordinates are only available if binning was performed over 'phase'
        plot_options[key_plot]['dim_plot']='phase' 

        #Print information
        plot_options[key_plot]['verbose']=True 

    #--------------------------------------   
    if ('prop_' in key_plot):

        #Abscissa
        plot_options[key_plot]['prop_DI_absc']='phase'                
        plot_options[key_plot]['prop_Intr_absc']='phase'
        
        #Invert horizontal axis
        #    - useful for retrograde orbit to follow the orbit from left to right (and decreasing phase)
        plot_options[key_plot]['retro_orbit']=False  

        #Plot values for detected CCFs only
        plot_options[key_plot]['plot_det']=False 

        #Print min/max values (to adjust plot ranges)
        plot_options[key_plot]['plot_bounds']=False
        
        #Reference level
        plot_options[key_plot]['plot_ref'] = True            

        #Print and plot mean value and dispersion 
        plot_options[key_plot]['plot_disp']=True    

        #Plot master property
        plot_options[key_plot]['plot_Mout']=False
        plot_options[key_plot]['plot_Mloc']=False

        #Plot HDI subintervals, if available
        plot_options[key_plot]['plot_HDI']= False    

        #Use different symbols for transits (disks vs squares)
        plot_options[key_plot]['use_diff_symb']= False    
        
        #Empty symbols
        plot_options[key_plot]['empty_in'] = True
        plot_options[key_plot]['empty_all']= False
        plot_options[key_plot]['empty_det']= False   
 
        #Bin properties 
        plot_options[key_plot]['bin_val'] = {}

        #Orders for SNR
        plot_options[key_plot]['idx_SNR']={'HARPS':[49],'HARPN':[46],'CORALIE':[46],'ESPRESSO':[102,103],'ESPRESSO_MR':[39],'CARMENES_VIS':[40],'NIRPS_HE':[57]}
        plot_options[key_plot]['idx_num_SNR']={'HARPN':[46]}
        plot_options[key_plot]['idx_den_SNR']={'HARPN':[46]}              
 
        #Save a text file of residual RVs vs phase
        plot_options[key_plot]['save_RVres'] = False
 

        #Polynomial fit of ordina property vs abscissa property 
        plot_options[key_plot]['deg_prop_fit']={}

        #Multiplication/addition of sinusoidal component to the fit 
        plot_options[key_plot]['fit_sin']={}

        #Scaling value for errors on fitted data
        plot_options[key_plot]['set_err']=1. 
    
        #Points to be fitted
        #    - set instrument and visit to an empty list for its out-of-transit exposures to be fitted automatically, otherwise indicate specific exposures
        plot_options[key_plot]['idx_fit']={}

    #--------------------------------------
    if 'atm' in key_plot:
      
        #Scaling factor
        plot_options[key_plot]['sc_fact10'] = 6.

    #--------------------------------------
    if (key_plot in ['occulted_regions','system_view']):
        
        #Choice of spectral band for intensity
        #    - from the main planet transit properties
        plot_options[key_plot]['iband']=0 

    #--------------------------------------
    if 'map_' in key_plot:

        #Atmospheric signal type 
        if 'map_Atm' in key_plot:plot_options[key_plot]['pl_atm_sign']='Absorption'            

        #Margins   
        plot_options[key_plot]['margins']=[0.15,0.3,0.84,0.95]  

        #Font size
        plot_options[key_plot]['font_size']=18  

        #Reverse image
        #    - if set to True, map is orientated so that velocity is the vertical axis, and phase the horizontal axis
        plot_options[key_plot]['reverse_2D']=False

        #Overplot surface RV model along the full transit chord
        plot_options[key_plot]['theoRV_HR']=False

        #Overplot surface RV model along the full transit chord for an aligned orbit
        plot_options[key_plot]['theoRV_HR_align'] = False 

        #Overplot surface RV model at the phases of the observations 
        plot_options[key_plot]['plot_theoRV']=False

        #Overplot RV(pl/star) model 
        plot_options[key_plot]['theoRVpl_HR']=False

        #Plot zero line markers
        plot_options[key_plot]['plot_zermark']= True 
        
        #Color map
        if 'map_DI' in key_plot:plot_options[key_plot]['cmap']="jet" 
        if 'map_Res' in key_plot:plot_options[key_plot]['cmap']="jet"             
        elif 'map_Intr' in key_plot:plot_options[key_plot]['cmap']="afmhot_r" 
        elif 'map_Atm' in key_plot:plot_options[key_plot]['cmap']="winter"             
        
        #Map color range
        plot_options[key_plot]['v_range_all']={}
        
        #Ranges
        plot_options[key_plot]['x_range_all']={}
        plot_options[key_plot]['y_range_all']={}
        
        #Overplot measured RVs
        plot_options[key_plot]['plot_measRV'] = 'none'       

        #Plot global and in-transit indexes
        plot_options[key_plot]['plot_idx']=False              

    return plot_options





  
def ANTARESS_plot_settings(plot_settings,plot_dic,gen_dic,data_dic,glob_fit_dic,theo_dic):
    r"""**ANTARESS default plot settings.**
    
    Initializes ANTARESS plot settings with default values.  
    
    Args:
        TBD
    
    Returns:
        None
    
    """   
    
    ################################################################################################################    
    #%% Weighing master
    ################################################################################################################  
    if gen_dic['specINtype'] and (plot_dic['DImast']!=''):
        key_plot = 'DImast'
        
        #%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)


    ################################################################################################################  
    #%% Global spectral corrections
    ################################################################################################################          

    ################################################################################################################    
    #%%% Instrumental calibration estimates
    ################################################################################################################ 
    if (plot_dic['gcal']!='') or (plot_dic['gcal_ord']!=''):
        key_plot = 'gcal'
        
        #%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)

        #%%%% Plot best-fit exposure models in each order plot
        plot_settings[key_plot]['plot_best_exp'] = True

        #%%%% Normalize calibration profiles over all exposures
        plot_settings[key_plot]['norm_exp'] = False
        
        #%%%% Plot mean calibration if calculated
        plot_settings[key_plot]['mean_gdet']  = True

        #%%%% Range per order
        #    - unfitted points are not shown in automatic ranges
        plot_settings[key_plot]['x_range_ord'] = None
        plot_settings[key_plot]['y_range_ord'] = None          
        
       

    ##################################################################################################
    #%%% Tellurics
    ##################################################################################################        
        
    ################################################################################################################    
    #%%%% Telluric CCF
    ################################################################################################################  
    if gen_dic['specINtype'] and (plot_dic['tell_CCF']!=''):
        key_plot = 'tell_CCF'

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)

        #%%%%% Molecules to plot
        plot_settings[key_plot]['tell_species'] = gen_dic['tell_species']
        
        #%%%%% Plot dispersion  
        plot_settings[key_plot]['print_disp']=None 




    ################################################################################################################    
    #%%%% Telluric properties
    ################################################################################################################   
    if gen_dic['specINtype'] and (plot_dic['tell_prop']!=''):
        key_plot = 'tell_prop'
        
        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)

        #%%%%% Molecules to plot
        plot_settings[key_plot]['tell_species'] = gen_dic['tell_species']
  





    ##################################################################################################
    #%%% Flux balance corrections
    ##################################################################################################        
          
    ################################################################################################################    
    #%%%% Global scaling masters        
    #    - plotting global measured masters used in the flux balance correction and the flux scaling of the spectra
    #    - all raw spectra are plotted by default, before they are corrected
    #    - the measured master built from these spectra is also plotted by default
    ################################################################################################################
    if gen_dic['specINtype'] and (plot_dic['glob_mast']!=''):
        key_plot = 'glob_mast'

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)        

        #%%%%% Plot global master over all visits    
        plot_settings[key_plot]['glob_mast_all']=None 

        #%%%%% Plot master of each visit    
        plot_settings[key_plot]['glob_mast_vis']=True 

        


    ################################################################################################################    
    #%%%% Global flux balance (exposures)
    #    - relative to the mean level of each profile
    ################################################################################################################
    if gen_dic['specINtype'] and (plot_dic['Fbal_corr']!=''):   
        key_plot = 'Fbal_corr'

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)         

        #%%%%% Plot order indexes
        plot_settings[key_plot]['plot_idx_ord'] = True
        
        #%%%%% Indexes of exposures and bins to be plotted 
        plot_settings[key_plot]['ibin_plot'] = {}
          
        #%%%%% Overplot all exposures or offset them
        plot_settings[key_plot]['gap_exp']=0.        

        #%%%%% Strip range used for correction
        plot_settings[key_plot]['strip_corr'] = False

        #%%%%% Model spectral resolution 
        #    - in dlnw = dw/w, if x_range is defined
        plot_settings[key_plot]['dlnw_plot'] = 0.002
    

    


    ################################################################################################################    
    #%%%% Global DRS flux balance (exposures)
    ################################################################################################################
    if gen_dic['specINtype'] and (plot_dic['Fbal_corr_DRS']!=''):
        key_plot = 'Fbal_corr_DRS'

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 
        
  
    
  
    ################################################################################################################    
    #%%%% Global flux balance (visits)
    #    - relative to the mean level of each profile
    ################################################################################################################
    if gen_dic['specINtype'] and (plot_dic['Fbal_corr_vis']!=''):   
        key_plot = 'Fbal_corr_vis'

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)         

        #%%%%% Plot order indexes
        plot_settings[key_plot]['plot_idx_ord'] = True
        
        #%%%%% Indexes of bins to be plotted 
        plot_settings[key_plot]['ibin_plot'] = {}
          
        #%%%%% Overplot all profiles or offset them
        plot_settings[key_plot]['gap_exp']=0.        

        #%%%%% Strip range used for correction
        plot_settings[key_plot]['strip_corr'] = False

        #%%%%% Model spectral resolution 
        #    - in dlnw = dw/w, if x_range is defined
        plot_settings[key_plot]['dlnw_plot'] = 0.002   

    
    
    
    

    ################################################################################################################ 
    #%%%% Intra-order flux balance
    ################################################################################################################ 
    if gen_dic['specINtype'] and (plot_dic['Fbal_corr_ord']!=''):
        key_plot = 'Fbal_corr_ord' 

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)          



        
        
        
        
        
        
    ################################################################################################################ 
    #%%%% Temporal flux balance
    ################################################################################################################ 
    if gen_dic['specINtype'] and (plot_dic['Ftemp_corr']!=''):
        key_plot = 'Ftemp_corr'

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)  
        




                 



    ################################################################################################################ 
    #%%% Cosmics correction
    ################################################################################################################ 
    if gen_dic['specINtype'] and (plot_dic['cosm_corr']!=''):
        key_plot = 'cosm_corr'

        #%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)         

        #%%%% Cosmic marker
        plot_settings[key_plot]['markcosm'] = True  
        plot_settings[key_plot]['ncosm'] = True  

        #%%%% Plot orders with detected cosmics only  
        plot_settings[key_plot]['detcosm'] = True

        #%%%% Plot number of cosmics per order
        plot_settings[key_plot]['cosm_vs_ord']=True         











    ################################################################################################################ 
    #%%% Persistent peaks master
    ################################################################################################################ 
    if gen_dic['specINtype'] and (plot_dic['permpeak_corr']!=''):
        key_plot = 'permpeak_corr'

        #%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)          
        
        
        
        
        
        
        
        
        




    ################################################################################################################ 
    #%%% Fringing correction
    ################################################################################################################ 
    if gen_dic['specINtype'] and (plot_dic['fring_corr']!=''):
        key_plot = 'fring_corr'

        #%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)
        
        
        
        
        



        
    ##################################################################################################
    #%% Disk-integrated profiles 
    ##################################################################################################        
        
    ##################################################################################################
    #%%% 2D maps
    ##################################################################################################     
    
    ##################################################################################################
    #%%%% Original profiles
    #    - allows visualizing the exposures used to build the master, and the ranges excluded because of planetary contamination
    ##################################################################################################
    if (plot_dic['map_DI_prof']!=''):  
        key_plot = 'map_DI_prof'

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)

        #%%%%% Profiles to plot
        #    - 'sp_corr' : spectral profiles have been corrected
        #    - 'detrend' : profiles have been detrended
        #    - 'aligned' : profiles have been aligned, after being corrected if requested
        #    - 'scaled' : profiles have been scaled, after being corrected and aligned if requested
        plot_settings[key_plot]['step']='scaled'
        

        
        
        
        
    ##################################################################################################
    #%%%% Binned profiles
    ##################################################################################################
    if (plot_dic['map_DIbin']!=''):
        key_plot = 'map_DIbin'

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)
     
        
     
        


    ##################################################################################################
    #%%%% 1D converted profiles
    ##################################################################################################
    if (any('spec' in s for s in data_dic['DI']['type'].values())) and (plot_dic['map_DI_1D']!=''):
        key_plot = 'map_DI_1D'

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)
        
        
        

        


  
    ##################################################################################################
    #%%% Individual profiles
    ##################################################################################################       

    ##################################################################################################
    #%%%% Original spectra (correction steps) 
    #    - before and after spectral corrections 
    #    - in their input rest frame (typically heliocentric)
    ##################################################################################################
    if gen_dic['specINtype'] and (plot_dic['sp_raw']!=''):
        key_plot = 'DI_prof'

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)        

        #%%%%% Plot spectra at two chosen steps of the correction process
        plot_settings[key_plot]['plot_pre']='raw'
        plot_settings[key_plot]['plot_post']='all'

        #%%%%% Plot binned data and errors
        plot_settings[key_plot]['plot_bin'] = True
        plot_settings[key_plot]['plot_bin_err'] = True

        #%%%%% Data bin size (in A)
        plot_settings[key_plot]['bin_width'] = 10.

        #%%%%% Only plot exposures & orders with detected cosmics         
        plot_settings[key_plot]['det_cosm']= False
     
        #%%%%% Only plot exposures & orders with masked persistent pixels 
        plot_settings[key_plot]['det_permpeak']= False 

        #%%%%% Plot telluric spectrum
        #    - the one associated with the 'plot_post' spectra
        plot_settings[key_plot]['plot_tell'] = False

        #%%%%% Plot binned pixels used for the color balance fit
        plot_settings[key_plot]['plot_bins']= False

        #%%%%% Plot continuum used for persistent peak masking
        plot_settings[key_plot]['plot_contmax']=True 








    ##################################################################################################
    #%%%% Original transmission spectra (correction steps)
    #    - in the star rest frame, offset or not by the systemic velocity
    ##################################################################################################
    if gen_dic['specINtype'] and (plot_dic['trans_sp']!=''):
        key_plot = 'trans_sp'

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 
        
        #%%%%% Plot spectra at two chosen steps of the correction process
        plot_settings[key_plot]['plot_pre']='raw'
        plot_settings[key_plot]['plot_post']='all'

        #%%%%% Plot binned data and errors
        plot_settings[key_plot]['plot_bin'] = True
        plot_settings[key_plot]['plot_bin_err'] = True

        #%%%%% Data bin size (in A)
        plot_settings[key_plot]['bin_width'] = 10.

        #%%%%% Force order level to unity
        plot_settings[key_plot]['force_unity'] = False
        
        #%%%%% Offset between uncorrected / corrected data
        plot_settings[key_plot]['gap_exp'] = 0.03
        
        #%%%%% Path to correction
        #%%%%%     - leave empty to use last result from 'wig_vis_fit'
        plot_settings[key_plot]['wig_path_corr'] = {}  

        #%%%%% Plot mean value over exposures
        plot_settings[key_plot]['plot_disp'] = True   

        #%%%%% Vertical range for dispersion plot
        plot_settings[key_plot]['y_range_disp'] = {}




    




    ##################################################################################################
    #%%%% Original profiles (processing steps) 
    #    - in their original rest frame (typically heliocentric), with their model fit
    ##################################################################################################
    for key_plot in ['DI_prof','DI_prof_res']:
        if plot_dic[key_plot]!='':
    
            #%%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)                 

            #%%%%% Choose profiles to plot
            #    - 'raw' : input profiles
            plot_settings[key_plot]['step']='raw'  

            ##############################################################################
            #%%%%% Profile and its fit
            #    - ccfs are continuum-normalized for the purpose of the plot
            if (key_plot=='DI_prof'):  
                pass
            
            ##############################################################################
            #%%%%% Residuals between the profile and its fit
            if (key_plot=='DI_prof_res'):
                if (not gen_dic['fit_DI']):stop('Activate "gen_dic["fit_DI"]" to plot "plot_dic["DI_prof_res"]"')     
                



    ##################################################################################################
    #%%%% Binned profiles
    ##################################################################################################
    for key_plot in ['DIbin','DIbin_res']:
        if plot_dic[key_plot]!='':
    
            #%%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)  

            ##############################################################################
            #%%%%% Profile and its fit
            if (key_plot=='DIbin'):
                pass

            ##############################################################################
            #%%%%% Residuals between the profile and its fit
            if (key_plot=='DIbin_res'):
                if (not gen_dic['fit_DI_gen']):stop('Activate "gen_dic["fit_DI_gen"]" to plot "plot_dic["DIbin_res"]"') 


    
        
        

        

        
        
        
        
        
    ##################################################################################################
    #%%%% 1D converted spectra
    ##################################################################################################
    if (any('spec' in s for s in data_dic['DI']['type'].values())) and (plot_dic['sp_DI_1D']!=''):
        key_plot = 'sp_DI_1D' 

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)

 

    
  
    
  
    
        
        

  
    
  

    ##################################################################################################
    #%%%% Aligned profiles (all)
    #    - in star rest frame
    #    - all profiles from a given visit together
    #    - spectra may have been corrected for color balance or not
    #     CCF have not yet been scaled in flux, which is done within the routine to allow better comparing them (his requires the transit scaling routine to have been run so that continuum pixels are defined)
    ##################################################################################################   
    if (plot_dic['all_DI_data']!=''):  
        key_plot = 'all_DI_data'

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)        

        #%%%%% Data type
        plot_settings[key_plot]['data_type']='CCF' 
        
            
  
    
  
    
  
        
  
    
        
        
    ##################################################################################################
    #%% Stellar CCF mask
    ##################################################################################################        
        
    ##################################################################################################
    #%%% Spectrum
    #    - plotting spectrum used for CCF mask generation and associated properties
    ##################################################################################################
    for key_plot in ['DImask_spectra','Intrmask_spectra']:
        if plot_dic[key_plot]!='':

            #%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)  

            #%%%% Plot master spectrum and mask at chosen step
            plot_settings[key_plot]['step']='cont'      
            
            #%%%% Plot various spectra
            plot_settings[key_plot]['plot_raw'] = True
            plot_settings[key_plot]['plot_norm'] = True
            plot_settings[key_plot]['plot_norm_reg'] = True

            #%%%% Print number of line selected in step
            plot_settings[key_plot]['print_nl']=True   

            #%%%% Plot minimum telluric depth to be considered
            plot_settings[key_plot]['tell_depth_min'] = False

            ##############################################################################
            #%%%% Disk-integrated profiles
            if (key_plot=='DImask_spectra'):
                pass

            ##############################################################################
            #%%%% Intrinsic profile
            if (key_plot=='Intrmask_spectra'):
                pass




    ##################################################################################################
    #%%% Line depth range selection
    ##################################################################################################
    for key_plot in ['DImask_ld','Intrmask_ld']:
        if plot_dic[key_plot]!='':

            #%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)  

            #%%%% Number of bins in histograms
            plot_settings[key_plot]['dist_info'] = ['hist','cum_w']
            plot_settings[key_plot]['x_bins_par'] = 40
            plot_settings[key_plot]['x_log_hist'] = False
            plot_settings[key_plot]['y_bins_par'] = 40    
            plot_settings[key_plot]['y_log_hist'] = False
            plot_settings[key_plot]['x_range_hist'] = None
            plot_settings[key_plot]['y_range_hist'] = None      
            
            ##############################################################################
            #%%%% Disk-integrated profiles
            if (key_plot=='DImask_ld'):
                pass
    
            ##############################################################################
            #%%%% Intrinsic profile
            if (key_plot=='Intrmask_ld'):
                pass




    ##################################################################################################
    #%%% Line depth and width selection
    ##################################################################################################
    for key_plot in ['DImask_ld_lw','Intrmask_ld_lw']:
        if plot_dic[key_plot]!='':

            #%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)  

            #%%%% Number of bins in histograms
            plot_settings[key_plot]['dist_info'] = ['hist','cum_w']
            plot_settings[key_plot]['x_bins_par'] = 40
            plot_settings[key_plot]['x_log_hist'] = False
            plot_settings[key_plot]['y_bins_par'] = 40    
            plot_settings[key_plot]['y_log_hist'] = False
            plot_settings[key_plot]['x_range_hist'] = None
            plot_settings[key_plot]['y_range_hist'] = None
            
            ##############################################################################
            #%%%% Disk-integrated profiles
            if (key_plot=='DImask_ld_lw'):
                pass
    
            ##############################################################################
            #%%%% Intrinsic profile
            if (key_plot=='Intrmask_ld_lw'):
                pass    

        

        




    ##################################################################################################
    #%%% Line position selection
    ##################################################################################################
    for key_plot in ['DImask_RVdev_fit','Intrmask_RVdev_fit']:
        if plot_dic[key_plot]!='':

            #%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)  

            #%%%% Number of bins in histograms
            plot_settings[key_plot]['dist_info'] =['hist','cum_w']
            plot_settings[key_plot]['x_bins_par'] = 40
            plot_settings[key_plot]['x_log_hist'] = False
            plot_settings[key_plot]['x_range_hist'] = None
            
            ##############################################################################
            #%%%% Disk-integrated profiles
            if (key_plot=='DImask_RVdev_fit'):
                pass
    
            ##############################################################################
            #%%%% Intrinsic profile
            if (key_plot=='Intrmask_RVdev_fit'):
                pass    






    ##################################################################################################
    #%%% Telluric selection
    #    - plotting relative depth between telluric and stellar lines for CCF mask generation
    ##################################################################################################
    for key_plot in ['DImask_tellcont','Intrmask_tellcont']:
        if plot_dic[key_plot]!='':

            #%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 

            #%%%% Number of bins in histograms
            plot_settings[key_plot]['dist_info'] =['hist','cum_w']
            plot_settings[key_plot]['x_bins_par'] = 40
            plot_settings[key_plot]['x_log_hist'] = False
            plot_settings[key_plot]['x_range_hist'] = None

            ##############################################################################
            #%%%% Disk-integrated profiles
            if (key_plot=='DImask_tellcont'):
                pass
    
            ##############################################################################
            #%%%% Intrinsic profile
            if (key_plot=='Intrmask_tellcont'):
                pass   
        

        


    ##################################################################################################
    #%%% VALD line depth correction
    ##################################################################################################
    for key_plot in ['DImask_vald_depthcorr','Intrmask_vald_depthcorr']:
        if plot_dic[key_plot]!='':

            #%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 

            #%%%% Figure size
            plot_settings[key_plot]['fig_size'] = (12,15)

            ##############################################################################
            #%%%% Disk-integrated profiles
            if (key_plot=='DImask_vald_depthcorr'):
                pass
    
            ##############################################################################
            #%%%% Intrinsic profile
            if (key_plot=='Intrmask_vald_depthcorr'):
                pass   







    ##################################################################################################
    #%%% Morphological (asymmetry) selection
    ##################################################################################################
    for key_plot in ['DImask_morphasym','Intrmask_morphasym']:
        if plot_dic[key_plot]!='':

            #%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)        

            #%%%% Number of bins in histograms
            plot_settings[key_plot]['dist_info'] =['hist','cum_w']
            plot_settings[key_plot]['x_bins_par'] = 40
            plot_settings[key_plot]['x_log_hist'] = False
            plot_settings[key_plot]['y_bins_par'] = 40    
            plot_settings[key_plot]['y_log_hist'] = False
            plot_settings[key_plot]['x_range_hist'] = None
            plot_settings[key_plot]['y_range_hist'] = None

            ##############################################################################
            #%%%% Disk-integrated profiles
            if (key_plot=='DImask_morphasym'):
                pass
    
            ##############################################################################
            #%%%% Intrinsic profile
            if (key_plot=='Intrmask_morphasym'):
                pass   
    



    ##################################################################################################
    #%%% Morphological (shape) selection
    ##################################################################################################
    for key_plot in ['DImask_morphshape','Intrmask_morphshape']:
        if plot_dic[key_plot]!='':

            #%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)       

            #%%%% Number of bins in histograms
            plot_settings[key_plot]['dist_info'] =['hist','cum_w']
            plot_settings[key_plot]['x_bins_par'] = 40
            plot_settings[key_plot]['x_log_hist'] = False
            plot_settings[key_plot]['y_bins_par'] = 40    
            plot_settings[key_plot]['y_log_hist'] = False
            plot_settings[key_plot]['x_range_hist'] = None
            plot_settings[key_plot]['y_range_hist'] = None

            ##############################################################################
            #%%%% Disk-integrated profiles
            if (key_plot=='DImask_morphshape'):
                pass
    
            ##############################################################################
            #%%%% Intrinsic profile
            if (key_plot=='Intrmask_morphshape'):
                pass   

        

        

    ##################################################################################################
    #%%% RV dispersion selection
    ##################################################################################################
    for key_plot in ['DImask_RVdisp','Intrmask_RVdisp']:
        if plot_dic[key_plot]!='':

            #%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)        

            #%%%% Number of bins in histograms
            plot_settings[key_plot]['dist_info'] =['hist','cum_w']
            plot_settings[key_plot]['x_bins_par'] = 40
            plot_settings[key_plot]['x_log_hist'] = False
            plot_settings[key_plot]['y_bins_par'] = 40    
            plot_settings[key_plot]['y_log_hist'] = False
            plot_settings[key_plot]['x_range_hist'] = None
            plot_settings[key_plot]['y_range_hist'] = None

            ##############################################################################
            #%%%% Disk-integrated profiles
            if (key_plot=='DImask_RVdisp'):
                pass
    
            ##############################################################################
            #%%%% Intrinsic profile
            if (key_plot=='Intrmask_RVdisp'):
                pass   















    ##################################################################################################
    #%% Properties of raw data and disk-integrated CCFs
    #    - possibility to plot them as a function of many variable to search for correlations
    ##################################################################################################
    if (plot_dic['prop_DI']!=''):
        
        #%%% Ordina properties
        #    - choose values to plot in ordina (list of properties)
        #    - we set all properties to be plotted as default so that their associated plot properties can be defined here
        #    - properties:
        # + 'rv' : centroid of the raw CCFs in heliocentric rest frame (in km/s)
        # + 'FWHM': width of raw CCFs (in km/s)
        # + 'ctrst': contrast of raw CCFs
        # + 'rv_l2c': RV(lobe)-RV(core) of double gaussian components
        # + 'FWHM_l2c': FWHM(lobe)/FWHM(core) of double gaussian components
        # + 'amp_l2c': contrast(lobe)/contrast(core) of double gaussian components
        # + 'rv_res' residuals from Keplerian curve (m/s)  
        # + 'RVdrift' : RV drift of the spectrograph, derived from the Fabry-Perot (m/s)
        #    + 'phase' : orbital phase
        #    + 'mu' : mu
        #    + 'lat' : stellar latitude 
        #    + 'lon' : stellar longitude 
        #    + projected position in the stellar frame
        # x (along the equator) : 'x_st'
        # y (along the spin axis) : 'y_st' 
        #    + 'AM' : airmass : 
        #    + 'seeing' : seeing : 
        #    + 'snr' : SNR : 
        # in that case select the indices of orders over which average the SNR
        #    + 'snr_R' : SNR ratio 
        # in that case select the indices of orders over which average the SNR for both numerator and denominator
        #    + 'colcorrmin', 'colcorrmax', 'colcorrR' : min/max color correction coefficients and ratio max/min 
        #      'colcorr450', 'colcorr550', 'colcorr650' : correction coefficients at the corresponding wavelengths (nm)
        #    + 'glob_flux_sc': ratio of total flux in each exposure profile, to their mean value, used to scale all profiles to the same global flux level
        #    + 'satur_check': check of saturation on detector
        #    + 'PSFx', 'PSFy' : sizes of PSF on detector (?)
        #      'PSFr' : average (quadratic) size
        #      'PSFang' : angle y/x in degrees  
        #    + coefficients of wiggles laws: wig_p_0, wig_p1, wig_wref, wig_ai[i=0,4]
        #    + 'alt': telescope altitude angle (deg)
        #    + 'ha','na','ca','s','rhk': activity indexes 
        #    + 'ADC1 POS','ADC1 RA','ADC1 DEC','ADC2 POS','ADC2 RA','ADC2 DEC': ESPRESSO ADC intel
        #    + 'TILT1 VAL1','TILT1 VAL2','TILT2 VAL1','TILT2 VAL2': ESPRESSO piezo intel
        #    + 'EW': equivalent width
        #    + 'biss_span': bissector span
        plot_settings['prop_DI_ordin']=['rv','rv_pip','rv_res','rv_pip_res','RVdrift','rv_l2c','RV_lobe',\
                                        'FWHM','FWHM_pip','FWHM_voigt','FWHM_l2c','FWHM_lobe','FWHM_ord0__IS__VS_','EW','vsini',\
                                        'ctrst','ctrst_pip','true_ctrst','ctrst_ord0__IS__VS_','amp','amp_l2c','amp_lobe','area',\
                                        'cont','c1_pol','c2_pol','c3_pol','c4_pol',\
                                        'biss_span','ha','na','ca','s','rhk',\
                                        'PC0','PC1','PC2','PC3','PC4','PC5','PC6','PC7','PC8','PC9','PC10',\
                                        'phase','mu','lat','lon','x_st','y_st',\
                                        'AM','flux_airmass','seeing','snr','snr_quad','snr_R','colcorrmin', 'colcorrmax', 'colcorrR','colcorr450', 'colcorr550', 'colcorr650',\
                                        'PSFx', 'PSFy','PSFr','PSFang','alt',\
                                        'glob_flux_sc','satur_check','ADC1 POS','ADC1 RA','ADC1 DEC','ADC2 POS','ADC2 RA','ADC2 DEC',\
                                        'TILT1 VAL1','TILT1 VAL2','TILT2 VAL1','TILT2 VAL2']     
        
        #%%% Settings for selected properties
        for plot_prop in plot_settings['prop_DI_ordin']:
            key_plot = 'prop_DI_'+plot_prop 

            #%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)               

            #%%%% Print and plot mean value and dispersion 
            plot_settings[key_plot]['disp_mod']='out' 
            
            #%%%% Save dispersion values for analysis in external routine
            plot_settings[key_plot]['save_disp']=False
            
            #%%%% Plot in-transit symbols in a different color
            plot_settings[key_plot]['col_in']=''

            #%%%% Overplot transit duration from system properties 
            plot_settings[key_plot]['plot_T14'] = False
  
            #%%%% Normalisation of values by their out-of-transit mean
            plot_settings[key_plot]['norm_ref']=False
    
            #%%%% RV plot
            if (plot_prop=='rv'):

                #%%%%% Plot theoretical RV curve
                plot_settings[key_plot]['theoRV'] = True                
    
            #%%%% RV pipeline plot
            if (plot_prop=='rv_pip'):pass 
            
            #%%%% RV residual plot 
            if (plot_prop=='rv_res'):pass

            #%%%% RV pipeline residual plot
            if (plot_prop=='rv_pip_res'):pass
                
            #%%%% FWHM plot
            if (plot_prop=='FWHM'):pass
            
            #%%%% FWHM pipeline plot
            if (plot_prop=='FWHM_pip'):pass
            
            #%%%% Contrast plot
            if (plot_prop=='ctrst'):pass
            
            #%%%% Contrast pipeline plot
            if (plot_prop=='ctrst_pip'):pass
            
            #%%%% Amplitude plot
            if (plot_prop=='amp'):pass

            #%%%% Area plot
            if (plot_prop=='area'):pass
            
            #%%%% Airmass plot
            if (plot_prop=='AM'):pass
            
            #%%%% Seeing plot
            if (plot_prop=='seeing'):pass
            
            #%%%% SNR plot
            if (plot_prop=='snr') or (plot_prop=='snr_quad'):pass
  
            #%%%% RV drift plot
            if (plot_prop=='RVdrift'):pass
            
            #%%%% Color correction coefficients
            if (plot_prop=='colcorrmin'):pass                  
            if (plot_prop=='colcorrmax'):pass
            if (plot_prop=='colcorr450'):pass
            if (plot_prop=='colcorr550'):pass
            if (plot_prop=='colcorr650'):pass

            #%%%% Ratio of lobe FWHM to core FWHM
            if (plot_prop=='FWHM_l2c'):pass
            
            #%%%% Ratio of lobe contrast to core amplitude
            if (plot_prop=='amp_l2c'):pass

            #%%%% RV shift bewtween lobe and core gaussian RV centroid
            if (plot_prop=='rv_l2c'):pass
            
            #%%%% Lobe FWHM 
            if (plot_prop=='FWHM_lobe'):pass

            #%%%% Lobe amplitude
            if (plot_prop=='amp_lobe'):pass

            #%%%% Lobe RV
            if (plot_prop=='RV_lobe'):pass

            #%%%% True contrast
            if (plot_prop=='true_ctrst'):pass
    

    
    
        
        
        
        

        
        
    
    ################################################################################################################  
    #%% Light curves
    ################################################################################################################         
    
    ################################################################################################################    
    #%%% Input light curves
    #    - used to rescale the flux
    #    - one plot for each band, each instrument, all visits
    #    - overplotting all visits together is useful to quickly compare the exposure ranges, the detection of their local stellar profiles, and determine binning windows
    ################################################################################################################ 
    if (plot_dic['input_LC']!=''):
        key_plot = 'input_LC'

        #%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 
        
        #%%%% Plot exposure-averaged light curves used for scaling
        plot_settings[key_plot]['plot_LC_exp'] = True
        
        #%%%% Plot HR input light curves
        plot_settings[key_plot]['plot_LC_HR'] = True
        
        #%%%% Plot raw imported light curve, if available
        plot_settings[key_plot]['plot_LC_imp'] = True

        #%%%% Print visit names
        plot_settings[key_plot]['plot_vis']=True

        #%%%% Indexes of bands to plot
        #     - default is all
        plot_settings[key_plot]['idx_bands']=[]
        
        #%%%% Plot achromatic light curve
        #     - if chromatic light curves were used, will overwrite the plotting of the chromatic light curves over the 'idx_bands' band
        plot_settings[key_plot]['achrom']=False        

        #%%%% Gap between visits light curves  
        plot_settings[key_plot]['lc_gap']=None    




    ##################################################################################################
    #%%% Effective scaling light curves     
    #   - plotting effective light curves used to rescale the flux
    #   - for input spectra only
    #   - one plot for each instrument, each visit, for all wavelengths in a given list
    #   - these scaling light curves are only defined at the phase of observed exposures, and for the corresponding wavelength, which might differ slightly from one exposure to the next
    ##################################################################################################
    if (plot_dic['spectral_LC']!=''):
        key_plot = 'spectral_LC'
        
        #%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 

        #%%%% Wavelengths to plot (A)
        plot_settings[key_plot]['wav_LC']=[5500.]
        
        
        
        
    



        
    ################################################################################################################  
    #%% Residual profiles
    ################################################################################################################        
        
    ################################################################################################################  
    #%%% 2D maps
    ################################################################################################################  

    ################################################################################################################  
    #%%%% Original profiles 
    ################################################################################################################  
    if (plot_dic['map_Res_prof']!=''):
        key_plot = 'map_Res_prof'
        
        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 






    ################################################################################################################ 
    #%%% Individual profiles
    ################################################################################################################ 

    ################################################################################################################ 
    #%%%% Original profiles 
    #    - in the star rest frame 
    ################################################################################################################ 
    if (plot_dic['Res_prof']!=''):
        key_plot = 'Res_prof'
        
        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 

        #%%%%% Overplot estimates for local stellar profiles
        plot_settings[key_plot]['estim_loc']= False 

        #%%%%% Default mode for estimates of local stellar profiles
        plot_settings[key_plot]['mode_loc_data_corr'] = 'glob_mod'

        #%%%%% Model from the global fit to all CCFs ('global')
        plot_settings[key_plot]['fit_type']='global'      
 


    ##################################################################################################
    #%%% PCA results
    ##################################################################################################
    if (plot_dic['pca_ana']!=''):        
        key_plot = 'pca_ana'

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 
        
        #%%%%% Subplots
        plot_settings[key_plot]['pc_var'] = True 
        for key in ['pc_rms','pc_bic','pc_hist','pc_prof','fft_prof']:plot_settings[key_plot][key] = False

        #%%%%% PC variances to overplot
        plot_settings[key_plot]['var_list'] = ['pre','post','out']

        #%%%%% Principal components profiles to plot
        plot_settings[key_plot]['pc_list'] = [0]
        plot_settings[key_plot]['pc_col'] = ['dodgerblue']

        #%%%%% FFT profiles to plot
        #    - from original residual profiles ('res'), corrected residual profiles ('corr'), or bootstrapped corrected residual profiles ('boot') 
        plot_settings[key_plot]['fft_list'] = ['res','corr','boot']

        #%%%%% Bornes du plot  
        for key in ['x_range_var','y_range_var','x_range_rms','y_range_rms','x_range_bic','y_range_bic','x_range_hist','y_range_hist','x_range_pc','y_range_pc','x_range_pc','y_range_pc','x_range_fft','y_range_fft']:plot_settings[key_plot][key]=None
         







    ##################################################################################################
    #%%% Residual dispersion
    #    - standard deviation with bin size for out-of-transit residual CCFs
    #    - one plot per exposure
    ##################################################################################################
    if (plot_dic['scr_search']!=''):
        key_plot = 'scr_search'
        
        #%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 









            
            
            
            
        
    ##################################################################################################
    #%% Intrinsic profiles
    ##################################################################################################        

    ##################################################################################################
    #%%% 2D maps
    ################################################################################################## 
        
    ##################################################################################################
    #%%%% Original profiles
    ##################################################################################################
    if (plot_dic['map_Intr_prof']!=''):
        key_plot = 'map_Intr_prof'

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 

        #%%%%% Plot aligned profiles
        plot_settings[key_plot]['aligned']=False


            

    ##################################################################################################
    #%%%% Binned profiles
    ##################################################################################################
    if (plot_dic['map_Intrbin']!=''):
        key_plot = 'map_Intrbin'

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 


    ##################################################################################################
    #%%%% 1D converted spectra
    ##################################################################################################
    if (plot_dic['map_Intr_1D']!=''):
        key_plot = 'map_Intr_1D'

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 


    ##################################################################################################
    #%%%% Estimates and residuals
    ##################################################################################################
    for key_plot in ['map_Intr_prof_est','map_Intr_prof_res']:
        if plot_dic[key_plot]!='':

            #%%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 

            #%%%%% Model to retrieve
            plot_settings[key_plot]['mode_loc_data_corr'] = 'glob_mod'

            ##############################################################################
            #%%%%% Estimates
            if key_plot=='map_Intr_prof_est':

                #%%%%%% Model always required
                plot_settings[key_plot]['plot_line_model'] = True
        
            ##############################################################################
            #%%%%% Residuals
            if key_plot=='map_Intr_prof_res':

                #%%%%%% Include out-of-transit residual profiles to the plot
                plot_settings[key_plot]['show_outres']=True
    
                #%%%%%% Correct only for continuum level
                plot_settings[key_plot]['cont_only']=True
                plot_settings[key_plot]['plot_line_model'] = False
                plot_settings[key_plot]['st_cont']=None
                
        

 
    ################################################################################################################ 
    #%%%% PC-based noise model 
    ################################################################################################################   
    if (plot_dic['map_pca_prof']!=''):
        key_plot = 'map_pca_prof'

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)         

        #%%%%% Colormap
        plot_settings[key_plot]['cmap'] = 'Spectral'









            
    ##################################################################################################
    #%%% Individual profiles
    ##################################################################################################              
            
    ##################################################################################################
    #%%%% Original profiles (grouped)
    #    - for a given visit
    ##################################################################################################
    if (plot_dic['all_intr_data']!=''):
        key_plot = 'all_intr_data'

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)         

        #%%%%% Data type
        plot_settings[key_plot]['data_type']='CCF' 







    ##################################################################################################
    #%%%% Original profiles
    ##################################################################################################
    for key_plot in ['Intr_prof','Intr_prof_res']:
        if plot_dic[key_plot]!='':

            #%%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)                    

            #%%%%% Model from the global fit to all profiles ('global')
            plot_settings[key_plot]['fit_type']='global'   

            ##############################################################################
            #%%%%% Flux profiles
            if (key_plot=='Intr_prof'):
                
                #%%%%%% Plot aligned profiles
                plot_settings[key_plot]['aligned']=False

            ##############################################################################
            #%%%%% Residuals profiles
            if (key_plot=='Intr_prof_res'):
                if not ((gen_dic['fit_Intr']) or (gen_dic['fit_IntrProf'])): 
                    if (not gen_dic['fit_Intr']):stop('Activate "gen_dic["fit_Intr"]" to plot "plot_dic["Intr_prof_res"]"')     
                    if (not gen_dic['fit_IntrProf']):stop('Activate "gen_dic["fit_IntrProf"]" to plot "plot_dic["Intr_prof_res"]"')     
                   
   



    ##################################################################################################
    #%%%% Binned profiles
    ##################################################################################################
    for key_plot in ['Intrbin','Intrbin_res']:
        if plot_dic[key_plot]!='':
    
            #%%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)  

            #%%%%% Plot reference level
            plot_settings[key_plot]['ref_level'] = False

            ##############################################################################
            #%%%%% Profile and its fit
            if (key_plot=='Intrbin'):pass

            ##############################################################################
            #%%%%% Residuals between the profile and its fit
            if (key_plot=='Intrbin_res'):pass


    


 

    ################################################################################################################   
    #%%%% 1D converted spectra
    ################################################################################################################   
    if (any('spec' in s for s in data_dic['Intr']['type'].values())) and ((plot_dic['sp_Intr_1D']!='') or (plot_dic['sp_Intr_1D_res']!='')):
        for key_plot in ['sp_Intr_1D','sp_Intr_1D_res']:
            if plot_dic[key_plot]!='':

                #%%%%% Generic settings
                plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)  
    
                ##############################################################################
                #%%%%% Profile and its fit
                if (key_plot=='sp_Intr_1D'):pass

                ##############################################################################
                #%%%%% Residuals between the profile and its fit
                if (key_plot=='sp_Intr_1D_res'):pass









    ##################################################################################################
    #%%% Chi2 over intrinsic property series
    ##################################################################################################
    if (plot_dic['chi2_fit_IntrProp']!=''):
        key_plot = 'chi2_fit_IntrProp'

        #%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)  

        #%%%% Threshold to identify and print outliers
        plot_settings[key_plot]['chi2_thresh']=3.  

        #%%%% Property
        plot_settings[key_plot]['prop'] = 'rv'

        #%%%% General path to the best-fit model to property series
        plot_settings[key_plot]['IntrProp_path']=None






    ################################################################################################################ 
    #%%% Range of planet-occulted properties
    ################################################################################################################ 
    if (plot_dic['plocc_ranges']!=''):
        key_plot = 'plocc_ranges'

        #%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)  

        #%%%% Choose values to plot among
        plot_settings[key_plot]['x_prop']='r_proj' 
        
        #%%%% Gap between exposures
        plot_settings[key_plot]['y_gap'] = 0.2
        
        #%%%% New bin ranges
        plot_settings[key_plot]['prop_bin']={}
        
        #%%%% Visit names
        plot_settings[key_plot]['plot_visid'] = True










    ################################################################################################################
    #%%% 1D PDFs from analysis of individual profiles
    ################################################################################################################
    for key_plot in ['prop_DI_mcmc_PDFs','prop_Intr_mcmc_PDFs']:
        if plot_dic[key_plot]!='':

            #%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)  

            #%%%% Figure size
            plot_settings[key_plot]['fig_size']=(10,5)

            #%%%% Choose property to plot
            plot_settings[key_plot]['plot_prop_list']=['rv']
            
            #%%%% Default MCMC settings
            plot_settings[key_plot]['nwalkers'] = 50
            plot_settings[key_plot]['nsteps'] = 1000
    
            #%%%% Number of subplots per row (>=1)
            plot_settings[key_plot]['nsub_col'] = 5

            #%%%% Spacing between subplots
            plot_settings[key_plot]['wspace'] = 0.05
            plot_settings[key_plot]['hspace'] = 0.05

            #%%%% Plot 1 sigma HDI or confidence intervals
            plot_settings[key_plot]['plot_conf_mode']=['HDI']

            #%%%% Tick intervals
            plot_settings[key_plot]['xmajor_int_all'] = {}
            plot_settings[key_plot]['xminor_int_all'] = {}

            #%%%% Common ranges
            plot_settings[key_plot]['x_range_all'] = {}
    
            ##############################################################################
            #%%%% Disk-integrated profiles
            if (key_plot=='prop_DI_mcmc_PDFs'):
                plot_settings[key_plot]['data_mode'] = 'DI'
                plot_settings[key_plot]['data_dic_idx'] = 'DI'

            ##############################################################################
            #%%%% Intrinsic profiles            
            if (key_plot=='prop_Intr_mcmc_PDFs'):
                plot_settings[key_plot]['data_mode'] = 'Intr'
                plot_settings[key_plot]['data_dic_idx'] = 'Res'
                



        




    ##################################################################################################
    #%%% Properties of planet-occulted stellar CCFs
    ##################################################################################################
    if (plot_dic['prop_Intr']!=''):

        #%%% Ordina properties
        #    - properties:
        # + 'rv' : centroid of the local stellar CCFs in the star rest frame (in km/s)
        # + 'rv_res' residuals from their RRM model (in km/s)        
        # + 'FWHM': width of the local CCFs (in km/s)
        # + 'ctrst': contrast of the local CCFs
        # + 'rv_l2c': RV(lobe)-RV(core) of double gaussian components
        # + 'FWHM_l2c': FWHM(lobe)/FWHM(core) of double gaussian components
        # + 'amp_l2c': contrast(lobe)/contrast(core) of double gaussian components
        plot_settings['prop_Intr_ordin']=['rv','rv_res','rv_l2c','RV_lobe','FWHM','FWHM_voigt','FWHM_l2c','FWHM_lobe','true_FWHM','ctrst','true_ctrst','amp','amp_l2c','amp_lobe','area','a_damp']

        #%%% Settings for selected properties
        for plot_prop in plot_settings['prop_Intr_ordin']:
            key_plot = 'prop_Intr_'+plot_prop 

            #%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 

            #%%%% Print and plot mean value and dispersion 
            plot_settings[key_plot]['disp_mod']='all'  

            #%%%% General path to the best-fit model to property series
            plot_settings[key_plot]['IntrProp_path']=None
                    
            #%%%% General path to the best-fit model to profile series
            plot_settings[key_plot]['IntrProf_path']=None
                
            #%%%% Default MCMC settings
            plot_settings[key_plot]['nwalkers'] = 50
            plot_settings[key_plot]['nsteps'] = 1000

            #%%%% Plot data-equivalent model from property fit 
            plot_settings[key_plot]['theo_obs_prop'] = False

            #%%%% Plot data-equivalent model from profile fit 
            plot_settings[key_plot]['theo_obs_prof'] = False

            #%%%% Plot high-resolution model from property fit
            plot_settings[key_plot]['theo_HR_prop'] = False

            #%%%% Plot high-resolution model from profile fit
            plot_settings[key_plot]['theo_HR_prof'] = False

            #%%%% Print system properties derived from common fit to all exposures (if relevant)
            plot_settings[key_plot]['plot_fit_comm']={}

            #%%%% RV plot
            if (plot_prop=='rv' ):
                
                #%%%%% Plot high-resolution model from nominal values in ANTARESS_systems.py
                plot_settings[key_plot]['theo_HR_nom'] = False            
                
                #%%%%% Overplot the different contributions
                plot_settings[key_plot]['contrib_theo_HR']= []
    
                #%%%%% Calculate model envelope from MCMC results (calculation of models at +-1 sigma range of the parameters)
                plot_settings[key_plot]['calc_envMCMC_theo_HR_nom']=''
            
                #%%%%% Calculate model sample from MCMC results (distribution of models following the PDF of the parameters)
                plot_settings[key_plot]['calc_sampMCMC_theo_HR_nom']=''   
                
                #%%%%% Predict local RVs measurements from nominal model with errors, and SNR of RMR signal
                plot_settings[key_plot]['predic']={}                
                
            #%%%% RV residual plot 
            if (plot_prop=='rv_res' ):

                #%%%%% Overplot the different contributions
                plot_settings[key_plot]['contrib_theo_HR']= []
    
                #%%%%% Subtract full model
                plot_settings[key_plot]['mod_compos'] = 'full'  

            #%%%% FWHM plot
            if (plot_prop=='FWHM' ):pass

            #%%%% Contrast plot
            if (plot_prop=='ctrst' ):pass
    
            #%%%% Damping coefficient
            if (plot_prop=='a_damp' ):pass
                
            #%%%% Lobe-core properties ratios
            if (plot_prop=='dgauss' in data_dic['Intr']['model'].values()):
        
                #%%%%% Ratio of lobe FWHM to core FWHM
                if (plot_prop=='FWHM_l2c' ):pass

                #%%%%% Ratio of lobe contrast to core amplitude
                if (plot_prop=='amp_l2c' ):pass

                #%%%%% RV shift between lobe and core gaussian RV centroid
                if (plot_prop=='rv_l2c' ):pass

            #%%%% True properties for complex models
            if (plot_prop!='gauss' in data_dic['Intr']['model'].values()):
                    
                #%%%%% True FWHM
                if (plot_prop=='true_FWHM' ):pass
    
                #%%%%% True contrast
                if (plot_prop=='true_ctrst' ):pass





    ################################################################################################################ 
    #%% Binned disk-integrated and intrinsic profile series
    #    - plotted together for comparison
    ################################################################################################################  
    if (plot_dic['binned_DI_Intr']!=''):
        key_plot = 'binned_DI_Intr'

        #%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 

        #%%%% Data type
        plot_settings[key_plot]['data_type']='CCF' 

        #%%%% Choose bin dimension for disk-integrated profiles
        plot_settings[key_plot]['dim_plot_DI']='phase' 

        #%%%% Choose bin dimension for intrinsic profiles
        plot_settings[key_plot]['dim_plot_intr']='r_proj' 











    ##################################################################################################
    #%% System view
    ##################################################################################################        
      
    ################################################################################################## 
    #%%% Occulted stellar regions
    #    - for a given system configuration, with RV of the planet-occulted regions
    ##################################################################################################  
    if (plot_dic['occulted_regions']!=''):
        key_plot = 'occulted_regions'

        #%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)         

        #%%%% Plot full orbit
        plot_settings[key_plot]['plot_orb']=True        

        #%%%% Ranges
        plot_settings[key_plot]['x_range'] = np.array([-1.15,1.15])
        plot_settings[key_plot]['y_range'] = np.array([-1.15,1.15])





    ################################################################################################################   
    #%%% Planetary system architecture
    #   - orbits of all planets in the system can be plotted in 3D
    #   - different views can be chosen
    #   - the stellar surface can show limb-darkening or limb-darkening-weighted radial velocity field
    ################################################################################################################ 
    if (plot_dic['system_view']!=''):
        key_plot = 'system_view'

        #%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)

        #%%%% Default planets to plot
        #    - set to transiting ones if undefined
        plot_settings[key_plot]['pl_to_plot'] = gen_dic['studied_pl']

        #%%%% Number of points in the planet orbits
        plot_settings[key_plot]['npts_orbits'] = np.repeat(10000,len(plot_settings[key_plot]['pl_to_plot'])) 

        #%%%% Position of planets along their orbit
        plot_settings[key_plot]['t_BJD'] = None
        plot_settings[key_plot]['GIF_generation'] = False
        plot_settings[key_plot]['xorp_pl'] = np.tile([[-0.5],[0.5]],len(plot_settings[key_plot]['pl_to_plot'])).T
        plot_settings[key_plot]['yorb_pl'] = np.repeat(0.5,len(plot_settings[key_plot]['pl_to_plot']))  

        #%%%% Position of oriented arrow along planet orbits
        plot_settings[key_plot]['xorb_dir'] = np.tile([[-2.],[2.]],len(plot_settings[key_plot]['pl_to_plot'])).T        
        plot_settings[key_plot]['yorb_dir'] = np.repeat(-0.5,len(plot_settings[key_plot]['pl_to_plot'])) 

        #%%%% Apparent size of the planet
        plot_settings[key_plot]['RpRs_pl'] = [data_dic['DI']['system_prop']['achrom'][pl_loc][0] for pl_loc in plot_settings[key_plot]['pl_to_plot']]
       
        #%%%% Orbit colors
        plot_settings[key_plot]['col_orb'] = np.repeat('forestgreen',len(plot_settings[key_plot]['pl_to_plot']))
        plot_settings[key_plot]['col_orb_samp'] = np.repeat('forestgreen',len(plot_settings[key_plot]['pl_to_plot']))
            
        #%%%% Number of orbits drawn randomly
        plot_settings[key_plot]['norb']=2500

        #%%%% Ranges of orbital parameters
        plot_settings[key_plot]['lambdeg_err']={}
        plot_settings[key_plot]['aRs_err']={}
        plot_settings[key_plot]['ip_err']={}
        plot_settings[key_plot]['b_range_all']={}            

        #%%%% Plot width
        plot_settings[key_plot]['width']=10.         

        #%%%% Choose view
        plot_settings[key_plot]['conf_system']='sky_ste'

        #%%%% Reference planet for the 'sky_orb' configuration
        plot_settings[key_plot]['pl_ref'] = plot_settings[key_plot]['pl_to_plot'][0]
        
        #%%%% Overlaying grid cell boundaries
        plot_settings[key_plot]['st_grid_overlay']=False
        plot_settings[key_plot]['pl_grid_overlay']=False
        
        #%%%% Number of cells on a diameter of the star and planets (must be odd)
        plot_settings[key_plot]['n_stcell']=theo_dic['nsub_Dstar']
        plot_settings[key_plot]['n_plcell']={}
        for pl_loc in plot_settings[key_plot]['pl_to_plot']:plot_settings[key_plot]['n_plcell'][pl_loc] = theo_dic['nsub_Dpl'][pl_loc]

        #%%%% Color stellar disk with RV, with limb-darkened specific intensity, with gravity-darkened specific intensity, or total flux
        plot_settings[key_plot]['disk_color']='RV'

        #%%%% Plot colorbar
        plot_settings[key_plot]['plot_colbar']=True 

        #%%%% Plot visible stellar equator
        plot_settings[key_plot]['plot_equ_vis']=True  
    
        #%%%% Position of oriented arrow along stellar equator
        plot_settings[key_plot]['xst_dir'] = [-0.1,0.1]  
        plot_settings[key_plot]['yst_dir'] = 0.
    
        #%%%% Plot hidden equator
        plot_settings[key_plot]['plot_equ_hid']=  False

        #%%%% Plot hidden stellar spin
        plot_settings[key_plot]['plot_stspin_hid']=  False
        
        #%%%% Plot stellar spin
        plot_settings[key_plot]['plot_stspin']=True 
        
        #%%%% Plot stellar poles
        plot_settings[key_plot]['plot_poles']=True  
        plot_settings[key_plot]['plot_hidden_pole']= False 
        
        #%%%% Plot stellar spots
        plot_settings[key_plot]['plot_spots'] = False
        
        #%%%% Number of positions of the spots to be plotted, equally distributed within the given time range.
        plot_settings[key_plot]['n_image_spots'] = 15

        #%%%% Use stellar rotation period to distribute the positions, instead of time
        plot_settings[key_plot]['plot_spot_all_Peq'] = True  

        #%%%%  Whether we want to show the track of the spots (spot_overlap=True) or the location of the spots (spot_overlap=False).
        # Only activated if we plot multiple exposures.
        plot_settings[key_plot]['spot_overlap'] = False
    
        #%%%% Overlay to the RV-colored disk a shade controlled by flux
        plot_settings[key_plot]['shade_overlay']=True       
        
        #%%%% Number of equi-RV curves
        plot_settings[key_plot]['n_equi']=None     
    
        #%%%% Color range
        plot_settings[key_plot]['val_range'] = None       
    
        #%%%% Overlay axis of selected frame
        plot_settings[key_plot]['axis_overlay']=False      
    
        #%%%% Plot axis boundaries
        plot_settings[key_plot]['margins']=[0.12,0.15,0.85,0.88]    
    
        #%%%% Plot boundaries
        plot_settings[key_plot]['x_range'] = np.array([-1.5,1.5])   
        plot_settings[key_plot]['y_range'] = np.array([-1.5,1.5]) 
        
        #%%%% RV range
        plot_settings[key_plot]['rv_range'] = None

        #%%%% GIF generation when using multiple exposures
        plot_settings[key_plot]['GIF_generation'] = False
    













    ##################################################################################################
    #%% Atmospheric profiles
    ##################################################################################################        

    ##################################################################################################
    #%%% 2D maps
    ################################################################################################## 
  
    ##################################################################################################
    #%%%% Original profiles
    ##################################################################################################
    if (plot_dic['map_Atm_prof']!=''):
        key_plot = 'map_Atm_prof'  

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 

        #%%%%% Plot profiles in star (False) or planet rest frame (True)
        plot_settings[key_plot]['aligned']=False






    ##################################################################################################
    #%%%% Binned profiles
    ##################################################################################################
    if (plot_dic['map_Atmbin']!=''):
        key_plot = 'map_Atmbin'  

        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)         


            
            

    ##################################################################################################
    #%%%% 1D converted spectra
    ##################################################################################################
    if (plot_dic['map_Atm_1D']!=''):
        key_plot = 'map_Atm_1D' 
        
        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 



    
    
    
    
    
    ##################################################################################################
    #%%% Individual profiles
    ##################################################################################################        
    

    ##################################################################################################
    #%%%% Original profiles (grouped)
    #    - for a given visit
    ################################################################################################## 
    if (plot_dic['all_atm_data']!=''):
        key_plot = 'all_atm_data'
                
        #%%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic) 

        #%%%%% Data type
        plot_settings[key_plot]['data_type']='CCF' 
        





    ##################################################################################################
    #%%%% Original profiles
    ##################################################################################################
    for key_plot in ['Atm_prof','Atm_prof_res']:
        if plot_dic[key_plot]!='':

            #%%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)                    

            #%%%%% Model from the global fit to all profiles ('global')
            plot_settings[key_plot]['fit_type']='global'   

            ##############################################################################
            #%%%%% Atmospheric profiles
            if (key_plot=='Atm_prof'):
                
                #%%%%%% Plot aligned profiles
                plot_settings[key_plot]['aligned']=False

            ##############################################################################
            #%%%%% Residuals profiles
            if (key_plot=='Atm_prof_res'):
                if not ((gen_dic['fit_Atm']) or (gen_dic['fit_AtmProf'])): 
                    if (not gen_dic['fit_Atm']):stop('Activate "gen_dic["fit_Atm"]" to plot "plot_dic["Atm_prof_res"]"')     
                    if (not gen_dic['fit_AtmProf']):stop('Activate "gen_dic["fit_AtmProf"]" to plot "plot_dic["Atm_prof_res"]"')   
                        







    ##################################################################################################
    #%%%% Binned profiles
    ##################################################################################################
    for key_plot in ['Atmbin','Atmbin_res']:
        if plot_dic[key_plot]!='':
    
            #%%%%% Generic settings
            plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)  

            ##############################################################################
            #%%%%% Profile and its fit
            if (key_plot=='Atmbin'):pass

            ##############################################################################
            #%%%%% Residuals between the profile and its fit
            if (key_plot=='Atmbin_res'):pass








    ################################################################################################################   
    #%%%% 1D converted spectra
    ################################################################################################################   
    if (any('spec' in s for s in data_dic['Atm']['type'].values())) and ((plot_dic['sp_Atm_1D']!='') or (plot_dic['sp_Atm_1D_res']!='')):
        for key_plot in ['sp_Atm_1D','sp_Atm_1D_res']:
            if plot_dic[key_plot]!='':

                #%%%%% Generic settings
                plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)  
    
                ##############################################################################
                #%%%%% Profile and its fit
                if (key_plot=='sp_Atm_1D'):pass

                ##############################################################################
                #%%%%% Residuals between the profile and its fit
                if (key_plot=='sp_Atm_1D_res'):pass







    ##################################################################################################
    #%%% Chi2 over atmospheric property series
    ##################################################################################################
    if (plot_dic['chi2_fit_AtmProp']!=''):
        key_plot = 'chi2_fit_AtmProp'

        #%%%% Generic settings
        plot_settings=gen_plot_default(plot_settings,key_plot,plot_dic,gen_dic)  

        #%%%% Threshold to identify and print outliers
        plot_settings[key_plot]['chi2_thresh']=3.  

        #%%%% Property
        plot_settings[key_plot]['prop'] = 'rv'

        #%%%% General path to the best-fit model to property series
        plot_settings[key_plot]['AtmProp_path']=None        
        
        
        
        
        


        
    return plot_settings


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import numpy as np
from copy import deepcopy
import bindensity as bind
from scipy.interpolate import interp1d
from pathos.multiprocessing import Pool
from ..ANTARESS_general.constant_data import c_light
from ..ANTARESS_conversions.ANTARESS_binning import pre_calc_bin_prof,weights_bin_prof
from ..ANTARESS_grids.ANTARESS_coord import excl_plrange
from ..ANTARESS_process.ANTARESS_data_process import calc_Intr_mean_cont
from ..ANTARESS_corrections.ANTARESS_detrend import corr_length_determination
from ..ANTARESS_general.utils import stop,dataload_npz,datasave_npz,MAIN_multithread,np_where1D,init_parallel_func,gen_specdopshift,check_data


def init_conversion(data_type_gen,gen_dic,prop_dic,inst,vis,mode,dir_save,data_dic):
    r"""**Initialization for conversion routines.**
    
    Initializes various parameters for conversion into CCF or S1D spectra. 
    
    Args:
        TBD
    
    Returns:
        None
    
    """     
    txt_print = {'CCFfromSpec':'CCF','1Dfrom2D':'1D'}[mode]
    if data_type_gen in ['DI','Intr']:    
        data_type = data_type_gen
        if data_type_gen=='Intr':
            iexp_conv = list(gen_dic[inst][vis]['idx_out'])+list(gen_dic[inst][vis]['idx_in'][prop_dic[inst][vis]['idx_def']])    #Global indexes
            data_type_key = ['Res','Intr']
            print('   > Converting OT residual and intrinsic spectra into '+txt_print)
        else:iexp_conv = range(data_dic[inst][vis]['n_in_visit'])    #Global indexes
    if data_type_gen in ['DI','Atm']:  
        if data_type_gen=='Atm': 
            data_type = prop_dic['pl_atm_sign']
            iexp_conv = prop_dic[inst][vis]['idx_def']    #Global or in-transit indexes
        data_type_key = [data_type_gen]
        print('   > Converting '+gen_dic['type_name'][data_type]+' spectra into '+txt_print)
    for key in data_type_key:dir_save[key] = gen_dic['save_data_dir']+key+'_data/'+mode+'/'+gen_dic['add_txt_path'][key]+'/'+inst+'_'+vis+'_'    
    
    return iexp_conv,data_type_key,data_type


def CCF_from_spec(data_type_gen,inst,vis,data_dic,gen_dic,prop_dic):
    r"""**Wrap-up function to compute CCFs** 
    
    Prepares spectral time-series for conversion into CCF, calling `new_compute_CCF()` 
    
    Args:
        TDB
    
    Returns:
        None
    
    """    
    data_vis=data_dic[inst][vis]
    gen_vis=gen_dic[inst][vis]
    dir_save = {}
    iexp_conv,data_type_key,_ = init_conversion(data_type_gen,gen_dic,prop_dic,inst,vis,'CCFfromSpec',dir_save,data_dic)

    #New paths
    #    - intrinsic and out-of-transit residual profiles are stored separately, contrary to global tables  
    flux_sc = False
    if data_type_gen in ['Intr','Atm']:
        dir_mast={}
        for gen in dir_save:dir_mast[gen] = {iexp_eff:dir_save[gen]+'ref_'+str(iexp_eff) for iexp_eff in data_vis['mast_'+gen+'_data_paths']}
        if gen_dic['flux_sc']:flux_sc = True
    proc_com_data_paths_new = gen_dic['save_data_dir']+'Processed_data/CCFfromSpec/'+inst+'_'+vis+'_com'

    #Calculating data
    if gen_dic['calc_'+data_type_gen+'_CCF']:
        print('         Calculating data')
        if data_vis['type']=='CCF':stop('Data must be spectral')         

        #Orders that are used to compute the CCFs
        ord_coadd = gen_dic[inst]['orders4ccf'] 
        nord_coadd = len(ord_coadd)

        #Calibration profile
        if data_vis['mean_gdet']:
            data_com = dataload_npz(data_vis['proc_com_data_paths'])
            mean_gdet_com = np.zeros([nord_coadd,data_com['dim_exp'][1]],dtype=float)
        else:gdet_ord=None
    
        #Upload data from all exposures 
        if flux_sc:data_scaling_all={}
        data_proc={}
        n_exp = len(iexp_conv) 
        for iexp_sub,iexp in enumerate(iexp_conv):
            gen = deepcopy(data_type_gen)
            
            #Retrieving data
            #    - out-of-transit residual profiles are retrieved with global indexes
            #    - in-transit intrinsic profiles are retrieved with in-transit indexes
            iexp_eff = deepcopy(iexp)     
            if (gen=='Intr'):
                if (iexp in gen_vis['idx_in']):iexp_eff = gen_vis['idx_exp2in'][iexp]    
                else:gen = 'Res'
            data_exp = dataload_npz(data_vis['proc_'+gen+'_data_paths']+str(iexp_eff))
         
            #Check that planetary ranges were not yet excluded
            if (data_type_gen=='Intr') and (iexp in gen_vis['idx_in']) and ('Intr' in data_dic['Atm']['no_plrange']) and (iexp in data_dic['Atm'][inst][vis]['iexp_no_plrange']) and data_exp['plrange_exc']:
                stop('    Planetary ranges excluded too soon: re-run gen_dic["intr_data"] with gen_dic["Intr_CCF"]')
                
            #Upload data
            if iexp_sub==0:
                if flux_sc:data_proc['cen_bins'] = np.zeros([n_exp,nord_coadd,data_vis['nspec']],dtype=float)
                data_proc['edge_bins'] = np.zeros([n_exp,nord_coadd,data_vis['nspec']+1],dtype=float)
                data_proc['flux'] = np.zeros([n_exp,nord_coadd,data_vis['nspec']],dtype=float)
                data_proc['cov'] = np.zeros([n_exp,nord_coadd],dtype=object)
                data_proc['cond_def'] = np.zeros([n_exp,nord_coadd,data_vis['nspec']],dtype=bool)             
            for key in ['edge_bins','flux','cov','cond_def']:
                data_proc[key][iexp_sub] = data_exp[key][ord_coadd]  

            #Upload flux scaling
            #    - we compute the equivalent CCF of the broadband scaling for the propagation of broadband spectral scaling on disk-integrated profiles into weights
            #    - global flux scaling is not modified  
            #    - scaling profiles are retrieved with global indexes
            if flux_sc:
                data_proc['cen_bins'][iexp_sub] = data_exp['cen_bins'][ord_coadd]          
                data_scaling_all[iexp]=dataload_npz(data_vis['scaled_'+gen+'_data_paths']+str(iexp)) 

            #Mean calibration profile over processed exposures
            #    - due to the various shifts of the processed spectra from the input rest frame, calibration profiles are not equivalent for a given line between exposure
            #      to maintain the relative flux balance between lines when computing CCFs, we calculate a common calibration profile to all processes exposures
            if data_vis['mean_gdet']:
                mean_gdet_exp = dataload_npz(data_vis['mean_gdet_'+gen+'_data_paths'][iexp_eff])['mean_gdet'] 
                for isub_ord,iord in enumerate(ord_coadd):
                    mean_gdet_com_ord=bind.resampling(data_com['edge_bins'][iord], data_proc['edge_bins'][iexp_sub,isub_ord],mean_gdet_exp[iord], kind=gen_dic['resamp_mode'])/n_exp 
                    idx_def_loc = np_where1D(~np.isnan(mean_gdet_com_ord))
                    mean_gdet_com_ord[0:idx_def_loc[0]+1]=mean_gdet_com_ord[idx_def_loc[0]]
                    mean_gdet_com_ord[idx_def_loc[-1]:]=mean_gdet_com_ord[idx_def_loc[-1]]
                    mean_gdet_com[isub_ord]+=mean_gdet_com_ord
                    
            #Upload weighing disk-integrated master 
            #    - the master always remains either defined on the common table, or on a specific table different from the table of its associated exposure
            #    - the master is computed after DI spectra have been converted into CCFs, and thus need conversion only for later profile types
            if data_type_gen in ['Intr','Atm']:
                data_ref = dataload_npz(data_vis['mast_'+gen+'_data_paths'][iexp_eff])
                if iexp_sub==0:
                    nspec_mast = (data_ref['cen_bins'].shape)[1]
                    data_proc['edge_bins_ref'] = np.zeros([n_exp,nord_coadd,nspec_mast+1],dtype=float)
                    data_proc['flux_ref'] = np.zeros([n_exp,nord_coadd,nspec_mast],dtype=float)
                    data_proc['cov_ref'] = np.zeros([n_exp,nord_coadd],dtype=object)                
                for key in ['edge_bins','flux','cov']:
                    data_proc[key+'_ref'][iexp_sub] = data_ref[key][ord_coadd]               

        #Initialize CCF tables
        CCF_all = np.zeros([n_exp,1,data_vis['nvel']],dtype=float)
        cov_exp_ord = np.zeros([n_exp,nord_coadd],dtype=object)
        nd_cov_exp_ord=np.zeros([n_exp,nord_coadd],dtype=int) 
        if data_type_gen in ['DI','Intr']:
            CCF_mask_wav = gen_dic['CCF_mask_wav'][inst]
            CCF_mask_wgt = gen_dic['CCF_mask_wgt'][inst]
        elif data_type_gen=='Atm':
            CCF_mask_wav = data_dic['Atm']['CCF_mask_wav']
            CCF_mask_wgt = data_dic['Atm']['CCF_mask_wgt']
        if data_type_gen in ['Intr','Atm']:
            CCF_ref = np.zeros([n_exp,1,data_vis['nvel']],dtype=float)
            cov_ref_ord = np.zeros([n_exp,nord_coadd],dtype=object)            
            nd_cov_ref_ord=np.zeros([n_exp,nord_coadd],dtype=int)
        
        #Velocity tables and initializations
        #    - we create an artificial order in velocity table to keep the same structure as spectra
        if data_type_gen=='DI':   
            velccf = data_vis['velccf']
            edge_velccf = data_vis['edge_velccf']          
        elif data_type_gen in ['Intr','Atm']:
            velccf = data_vis['velccf_star']
            edge_velccf = data_vis['edge_velccf_star']
        cen_bins = np.tile(velccf,[1,1])
        edge_bins = np.tile(edge_velccf,[1,1])  
        
        #Flux scaling
        if flux_sc: 
            norm_loc_flux_scaling_CCF = np.zeros(n_exp,dtype=float)                  
            loc_flux_scaling_CCF = np.zeros([n_exp,data_vis['nvel']],dtype=float)   
            
        #Calculate CCF over requested orders in each exposure
        #    - the covariance of CCFs calculated over different orders may have different dimensions, thus we first store them independently before they can be co-added
        #    - the structure below works for both s1d and e2ds
        ord_coadd_eff = []
        for isub,iord in enumerate(ord_coadd):
 
            #Identify lines that can contribute to all exposures for current order
            idx_maskL_kept = check_CCF_mask_lines(n_exp,data_proc['edge_bins'][:,isub],data_proc['cond_def'][:,isub],CCF_mask_wav,edge_velccf)

            #Calculating CCF for current order in each exposure with contributing lines
            #    - parallelisation is disabled, as it is inefficient given the size of the tables to process
            if len(idx_maskL_kept)>0:
                ord_coadd_eff+=[isub]
                if data_vis['mean_gdet']:
                    gdet_ord = bind.resampling(data_proc['edge_bins'][iexp_sub,isub],data_com['edge_bins'][iord],mean_gdet_com[isub], kind=gen_dic['resamp_mode'])  
                    idx_def_loc = np_where1D(~np.isnan(gdet_ord))
                    gdet_ord[0:idx_def_loc[0]+1]=gdet_ord[idx_def_loc[0]]
                    gdet_ord[idx_def_loc[-1]:]=gdet_ord[idx_def_loc[-1]]
                for iexp_sub,iexp in enumerate(iexp_conv):                      
                    flux_ord,cov_ord = new_compute_CCF(data_proc['edge_bins'][iexp_sub,isub],data_proc['flux'][iexp_sub,isub],data_proc['cov'][iexp_sub,isub],gen_dic['resamp_mode'],edge_velccf,CCF_mask_wgt[idx_maskL_kept],CCF_mask_wav[idx_maskL_kept],1.,cal = gdet_ord)[0:2]
                    CCF_all[iexp_sub,0]+=flux_ord
                    cov_exp_ord[iexp_sub,isub] = cov_ord 
                    nd_cov_exp_ord[iexp_sub,isub] = np.shape(cov_ord)[0]

                    #Compute CCF of spectral scaling
                    #    - loc_flux_scaling = 1 - LC
                    #      loc_flux_scaling_CCF = X - CCF_LC
                    #       we thus calculate the CCF of 1 to get X and normalize the scaling profile
                    if flux_sc and (not data_scaling_all[iexp]['null_loc_flux_scaling']):                
                        loc_flux_scaling_CCF_exp,_,norm_loc_flux_scaling_CCF_exp = new_compute_CCF(data_proc['edge_bins'][iexp_sub,isub],data_scaling_all[iexp]['loc_flux_scaling'](data_proc['cen_bins'][iexp_sub,isub]),None,gen_dic['resamp_mode'],edge_velccf,CCF_mask_wgt[idx_maskL_kept],CCF_mask_wav[idx_maskL_kept],1.,cal = gdet_ord)
                        loc_flux_scaling_CCF[iexp_sub] +=loc_flux_scaling_CCF_exp
                        norm_loc_flux_scaling_CCF[iexp_sub] += norm_loc_flux_scaling_CCF_exp

                    #Computing CCF of master disk-integrated spectrum
                    #    - so that it can be used in the weighing profiles
                    if data_type_gen in ['Intr','Atm']:
                        flux_temp,cov_temp = new_compute_CCF(data_proc['edge_bins_ref'][iexp_sub,isub],data_proc['flux_ref'][iexp_sub,isub],data_proc['cov_ref'][iexp_sub,isub],gen_dic['resamp_mode'],edge_velccf,CCF_mask_wgt[idx_maskL_kept],CCF_mask_wav[idx_maskL_kept],1.,cal = gdet_ord)[0:2]
                        CCF_ref[iexp_sub,0]+=flux_temp
                        cov_ref_ord[iexp_sub,isub] = cov_temp 
                        nd_cov_ref_ord[iexp_sub,isub] = np.shape(cov_temp)[0]

        #Check
        if len(ord_coadd_eff)==0:stop('         No lines contribute to all exposures')

        #Computing final covariance matrix in artificial new order
        for iexp_sub,iexp in enumerate(iexp_conv):
            gen = deepcopy(data_type_gen)
            iexp_eff = deepcopy(iexp)    #out- (global) or in-transit index
            cond_def_exp = (~np.isnan(CCF_all[iexp_sub,0]))[None,:] 
            data_CCF_exp={}
            if (data_type_gen=='Intr'):
                if (iexp in gen_vis['idx_in']):
                    iexp_eff = gen_vis['idx_exp2in'][iexp] 
                    
                    #Set to nan planetary ranges in intrinsic CCFs
                    #    - this is not done for disk-integrated and residual CCFs, as planetary signals need to be kept in them to be later extracted (planetary ranges are temporarily excluded when analyzing CCFs from those profiles)
                    if ('Intr' in data_dic['Atm']['no_plrange']) and (iexp in data_dic['Atm'][inst][vis]['iexp_no_plrange']):
                        cond_in_pl = ~( np.ones(data_vis['nvel'],dtype=bool) & excl_plrange(cond_def_exp[0],data_dic['Atm'][inst][vis]['exclu_range_star'],iexp,edge_bins[0],'CCF')[0])
                        CCF_all[iexp_sub,0,cond_in_pl]=np.nan
                        cond_def_exp[0,cond_in_pl]=False  
                        data_CCF_exp['plrange_exc'] = True
                    else:data_CCF_exp['plrange_exc'] = False
                        
                else:gen = 'Res'

            #Maximum dimension of covariance matrix in current exposure from all contributing orders
            nd_cov_exp = np.amax(nd_cov_exp_ord[iexp_sub,:])
            if data_type_gen in ['Intr','Atm']:nd_cov_exp = np.max([nd_cov_exp,np.amax(nd_cov_ref_ord[iexp_sub,:])])  
           
            #Co-adding contributions from orders
            cov_exp = np.zeros(1,dtype=object)
            cov_exp[0] = np.zeros([nd_cov_exp,data_vis['nvel']])
            if data_type_gen in ['Intr','Atm']:
                cov_ref = np.zeros(1,dtype=object)
                cov_ref[0] = np.zeros([nd_cov_exp,data_vis['nvel']])
            for isub in ord_coadd_eff:
                cov_exp[0][0:nd_cov_exp_ord[iexp_sub,isub],:] +=  cov_exp_ord[iexp_sub,isub]   
                if data_type_gen in ['Intr','Atm']:cov_ref[0][0:nd_cov_ref_ord[iexp_sub,isub],:] +=  cov_ref_ord[iexp_sub,isub]   
      
            #Saving data for each exposure
            #    - CCF are stored independently of input spectra, so that both can be retrieved
            data_CCF_exp.update({'cen_bins':cen_bins,'edge_bins':edge_bins,'flux':CCF_all[iexp_sub],'cond_def':cond_def_exp,'cov':cov_exp,'nd_cov':nd_cov_exp})              
            datasave_npz(dir_save[gen]+str(iexp_eff),data_CCF_exp)
            
            #Processing disk-integrated masters
            if data_type_gen in ['Intr','Atm']:
                datasave_npz(dir_mast[gen][iexp_eff],{'cen_bins':cen_bins,'edge_bins':edge_bins,'flux':CCF_ref[iexp_sub],'cov':cov_ref})

            #Redefine spectral scaling table
            if flux_sc:
                
                #Normalizing scaling CCF
                #    - if 'null_loc_flux_scaling' then 'loc_flux_scaling_CCF' remains set to 0
                if (not data_scaling_all[iexp]['null_loc_flux_scaling']):loc_flux_scaling_CCF[iexp_sub,:]/=norm_loc_flux_scaling_CCF[iexp_sub]
                
                #Defining the scaling CCF as a function
                #    - if scaling light curves are chromtic we propagate the chromaticity into the scaling CCF
                if not data_scaling_all[iexp]['chrom']:loc_flux_scaling_exp = np.poly1d(np.mean(loc_flux_scaling_CCF[iexp_sub]))                
                else:loc_flux_scaling_exp = interp1d(cen_bins[0],loc_flux_scaling_CCF[iexp_sub],fill_value=(loc_flux_scaling_CCF[iexp_sub,0],loc_flux_scaling_CCF[iexp_sub,-1]), bounds_error=False)
                
                #Saving
                data_scaling_all[iexp]['loc_flux_scaling'] = loc_flux_scaling_exp
                data_scaling_all[iexp]['chrom'] = False
                datasave_npz(dir_save[gen]+'_scaling_'+str(iexp),data_scaling_all[iexp])

        #Update common tables
        #    - set to the table in the star rest frame, as it is the one that will be used in later operations                
        datasave_npz(proc_com_data_paths_new,{'dim_exp':[1,data_vis['nvel']],'nspec':data_vis['nvel'],'cen_bins':np.tile(data_vis['velccf_star'],[1,1]),'edge_bins':np.tile(data_vis['edge_velccf_star'],[1,1])})

    else:
        check_data({'path':proc_com_data_paths_new})

    #Updating path to processed data and checking it has been calculated
    data_vis['proc_com_data_paths'] = proc_com_data_paths_new
    for gen in dir_save:
        data_vis['proc_'+gen+'_data_paths'] = dir_save[gen]  
        if flux_sc:data_vis['scaled_'+gen+'_data_paths'] = dir_save[gen]+'_scaling_'
        if gen in ['Intr','Atm']:data_vis['mast_'+gen+'_data_paths'] = dir_mast[gen]

    #Convert spectral mode 
    #    - all operations afterwards will be performed on CCFs
    #    - tellurics are not propagated to calculate weights in CCF mode 
    #    - no spectral calibration is applied
    print('         ANTARESS switched to CCF processing')
    data_vis['comm_sp_tab']=True
    data_vis['tell_sp'] = False 
    data_vis['mean_gdet'] = False 
    data_vis['type']='CCF'
    data_vis['nspec'] = data_vis['nvel']
    data_dic[inst]['nord'] = 1
    data_vis['dim_all'] = [data_vis['n_in_visit'],data_dic[inst]['nord'],data_vis['nvel']]
    data_vis['dim_exp'] = [data_dic[inst]['nord'],data_vis['nvel']]
    data_vis['dim_ord'] = [data_vis['n_in_visit'],data_vis['nvel']]
    if ('chrom' in data_vis['system_prop']):
        data_vis['system_prop']['chrom_mode'] = 'achrom'
        data_vis['system_prop'].pop('chrom')

    return None    

def ResIntr_CCF_from_spec(inst,vis,data_dic,gen_dic):
    r"""**CCF conversion** 
    
    Wrap-up for conversion of out-of-transit residual and intrinsic spectra into CCFs.
    
    Args:
        TDB
    
    Returns:
        None
    
    """ 
    data_inst = data_dic[inst]
    data_vis = data_inst[vis]
    gen_vis = gen_dic[inst][vis]

    #Calculating CCFs
    #    - if there is atmospheric contamination, excluding them from spectra before the conversion of intrinsic profiles would create empty ranges that shift between exposures
    # due to the planet orbital motion, thus potentially excluding mask lines from some exposures and not others, and resulting in CCFs that are not equivalent between all exposures
    #      the exclusion is thus applied after the conversion, internally to the routine 
    CCF_from_spec('Intr',inst,vis,data_dic,gen_dic,data_dic['Intr'])

    #Continuum pixels over all exposures
    #    - exclusion of planetary ranges is not required for intrinsic profiles if already applied to their definition, and if not already applied contamination is either negligible or neglected  
    cond_def_cont_all  = np.zeros(data_inst[vis]['dim_all'],dtype=bool)
    for i_in,iexp in zip(gen_vis['idx_exp2in'],range(data_vis['n_in_visit'])): 
        if i_in ==-1:
            gen = 'Res'
            iexp_eff = iexp
        else:
            gen='Intr'
            iexp_eff = i_in
        data_exp = dataload_npz(data_vis['proc_'+gen+'_data_paths']+str(iexp_eff))

        #Continuum ranges
        #    - if planetary contamination is excluded from residual out-of-transit profiles, define a large enough initial continuum range if the planet has a wide velocimetric motion 
        for iord in range(data_dic[inst]['nord']):
            if iord in data_dic[gen]['cont_range'][inst]:
                for bd_int in data_dic[gen]['cont_range'][inst][iord]:
                    cond_def_cont_all[iexp,iord] |= ((data_exp['edge_bins'][iord,0:-1]>=bd_int[0]) & (data_exp['edge_bins'][iord,1:]<=bd_int[1]))        
            else:cond_def_cont_all[iexp,iord] = True
            cond_def_cont_all[iexp,iord] &= data_exp['cond_def'][iord]
            if (gen=='Res') and ('Res_prof' in data_dic['Atm']['no_plrange']) and (iexp in data_dic['Atm'][inst][vis]['iexp_no_plrange']):     
                cond_def_cont_all[iexp,iord] &= excl_plrange(cond_def_cont_all[iexp,iord],data_dic['Atm'][inst][vis]['exclu_range_star'],iexp,data_exp['edge_bins'][iord],'CCF')[0]

    #Definition of continuum pixels and errors
    if data_dic['Intr']['disp_err']:print('         Setting errors on intrinsic CCFs to continuum dispersion')
    for i_in,iexp in zip(gen_vis['idx_exp2in'],range(data_vis['n_in_visit'])): 
        if i_in ==-1:
            gen = 'Res'
            iexp_eff = iexp
        else:
            gen='Intr'
            iexp_eff = i_in
        data_exp = dataload_npz(data_vis['proc_'+gen+'_data_paths']+str(iexp_eff))
    
        #Definition of errors on based on the dispersion in the continuum
        #    - attributing constant error to all points, if requested
        #    - if atmospheric profiles are extracted this operation must be done on residual profiles, and not intrinsic ones, so that errors can then be propagated         
        if data_dic['Intr']['disp_err']: 

            #Continuum dispersion
            for iord in range(data_dic[inst]['nord']):
                disp_cont=data_exp['flux'][iord,cond_def_cont_all[iexp,iord]].std() 

                #Error table
                #    - scaled if requested
                err_tab = np.sqrt(gen_dic['g_err'][inst])*np.repeat(disp_cont,data_vis['nspec'])
                data_exp['cov'][iord] = (err_tab*err_tab)[None,:]

            #Overwrite exposure data
            np.savez_compressed(data_vis['proc_'+gen+'_data_paths']+str(iexp_eff),data=data_exp,allow_pickle=True)  

    #Updating/correcting continuum level          
    if data_dic['Intr']['calc_cont']:           
        data_dic['Intr'][inst][vis]['mean_cont']=calc_Intr_mean_cont(data_vis['n_in_tr'],data_dic[inst]['nord'],data_dic[inst][vis]['nspec'],data_vis['proc_Intr_data_paths'],data_vis['type'],data_dic['Intr']['cont_range'],inst,data_dic['Intr']['cont_norm'],gen_dic['flag_err_inst'][inst])
        np.savez_compressed(data_vis['proc_Intr_data_paths']+'_add',data={'mean_cont':data_dic['Intr'][inst][vis]['mean_cont']},allow_pickle=True)
    else:
        check_data({'0':data_vis['proc_Intr_data_paths']+'_add'},silent=True)
        data_dic['Intr'][inst][vis]['mean_cont'] = dataload_npz(data_vis['proc_Intr_data_paths']+'_add')['mean_cont']

    #Determine the correlation length for the visit
    if gen_dic['scr_search']:
        corr_length_determination(data_dic['Res'][inst][vis],data_vis,gen_dic[inst][vis]['scr_search'],inst,vis,gen_dic)    

    return None








def check_CCF_mask_lines(n_exp,edge_wav_all,cond_def_all,wav_mask,edge_velccf):
    r"""**CCF mask lines** 
    
    Identifies mask lines that should be used to compute CCFs.
    We keep lines for which the CCF RV table (once converted into wavelength space):
        
     - is fully within the spectral range of the input spectrum
     - contains no nan from the input spectrum  
     
    The check is made for the time-series of spectra given as input as a whole, as CCFs should be calculated from the same mask lines in all exposures. 
    Tables must have dimension [n_time x n_spec] as input.
    
    Args:
        TDB
    
    Returns:
        idx_lines_kept (1D array): indexes of lines kept in mask.
    
    """ 
    #Conversion from wavelengths in star rest rame (lines transitions) to wavelength in the spectrum rest frame
    #    - what we convert is the radial velocity table of the CCF (the edges of its bins)
    #      the new table has dimension n_lines*(n_RV+1)
    #      we later loop on each line, so that the sub-table is centered on the expected line transition, assuming the mask is centered on each successive pixel
    #    - nu_received/nu_source = (1 + v_receiver/c )/(1 - v_source/c )
    #  w_source/w_received   = (1 + v_receiver/c )/(1 - v_source/c )
    #  with v_receiver > 0 if receiver moving toward the source
    #  with v_source > 0   if source moving toward the receiver
    #    - with the receiver at rest in the frame where the spectrum was measured:
    #  v_receiver = 0.
    #  v_source = - rv_star_spframe since by definition rv < 0 when moving toward us
    #  -> wav_rstar/w_spframe = 1./(1 + rv_star_spframe/c )
    #  -> w_spframe = wav_rstar*(1 + rv_star_spframe/c ) 
    #    - dimension n_libes x n_RV
    edge_mask_lines=wav_mask[:,None]*(1.+(edge_velccf/c_light))

    #First start wavelength and last end wavelength of the RV table relative to each mask line
    #    - dimension n_lines
    wstart_mask_table = edge_mask_lines[:,0]
    wstop_mask_table = edge_mask_lines[:,-1]    
    
    #Cheking all input exposures
    n_lines = len(wav_mask)
    idx_lines_kept = np.arange(n_lines)
    for iexp in range(n_exp):

        #Check for discontinuous input table
        low_pix_wav = edge_wav_all[iexp,0:-1]
        high_pix_wav = edge_wav_all[iexp,1::]

        #Identifying the first and last original pixels overlapping with the RV table of each line in wavelength space
        #    - we exploit searchsorted(x,y) which returns i_k in x for each y[k] so that 
        # x_low[i_k-1]  <= y_low[k]  <  x_low[i_k]  (side = right)
        # x_high[i_k-1] <  y_high[k] <= x_high[i_k] (side = left)
        #    - with x the lower boundary of original pixels and y the lower boundary of new bins, we get i_k-1 the index of the first original pixel overlapping with the new bin
        #      with x the upper boundary of original pixels and y the upper boundary of new bins, we get i_k the index of the last original pixel overlapping with the new bin
        #    - only defined pixels must be given to "searchsorted"
        #    - y values beyond the first & last values in x returns indexes -1 or n
        #    - because the old and new bins must be continuous here we could use the edge tables
        #      however this would make searchsorted find the position of all new bins in the mask table, while here we only need to find the start and end pixel
        idx_first_overpix =np.searchsorted(low_pix_wav,wstart_mask_table,side='right')-1
        idx_last_overpix =np.searchsorted(high_pix_wav,wstop_mask_table,side='left')        
        
        #Keep lines whose range is fully within that of the input spectrum
        #    - condition can be invalid for lines at the edges of the spectral range and wide RV tables
        idx_keepL = np_where1D( (idx_first_overpix>=0) & (idx_last_overpix<=len(low_pix_wav)-1) )
    
        #Keep lines whose range does not contain nan values from the input spectrum
        cond_keepL_sub = np.ones(len(idx_keepL),dtype=bool)
        for iline,(idx_first_overpix_line,idx_last_overpix_line) in enumerate(zip(idx_first_overpix[idx_keepL],idx_last_overpix[idx_keepL])):
            cond_keepL_sub[iline] &= (np.sum(~cond_def_all[iexp,idx_first_overpix_line:idx_last_overpix_line+1]) == 0)
    
        #Updates indexes of lines to be kept
        idx_keepL=idx_keepL[cond_keepL_sub]           #indexes relative to reduced table
        idx_lines_kept = idx_lines_kept[idx_keepL]    #indexes relative to reduced table but containting original line indexes in mask
        
        #Updates mask tables to speed the check
        wstart_mask_table = wstart_mask_table[idx_keepL]
        wstop_mask_table = wstop_mask_table[idx_keepL]

    return idx_lines_kept






def new_compute_CCF(edge_wav,flux,cov,resamp_mode,edge_velccf,wght_mask,wav_mask,nthreads,cal = None):
    r"""**Main routine to compute CCF** 
    
    Calculates CCFs from input spectra, propagating covariance matrix
    
    Args:
        TDB
    
    Returns:
        None
    
    """ 
    #Check for discontinuous input table
    low_pix_wav = edge_wav[0:-1]
    high_pix_wav = edge_wav[1::]
    if np.any(low_pix_wav[1:]-high_pix_wav[0:-1]):
        stop('Spectral bins must be continuous')
    
    #Line wavelength at each requested RV
    #    - for each line, the rest wavelength of its transition (source, assimilated to the star rest frame) is shifted to all trial wavelengths of the spectrum rest frame (receiver) associated with the CCF RVs
    #      see gen_specdopshift():
    # w_receiver = w_source * (1+ rv[s/r]/c))
    # w_trial = w_line * (1+ rv[line/trial]/c))
    n_RV=len(edge_velccf)-1
    edge_mask_lines=wav_mask[:,None]*gen_specdopshift(edge_velccf)

    #Call to parallelized function   
    if (nthreads>1) and (nthreads<=len(wght_mask)):            
        common_args=(n_RV,edge_wav,flux,cov,cal,resamp_mode)
        chunkable_args=[edge_mask_lines,wght_mask]
        fluxCCF,covCCF_line,nd_covCCF_line,contCCF=multithread_new_compute_CCF(sub_new_compute_CCF,nthreads,len(wght_mask),chunkable_args,common_args)                           

    #Regular routine
    else:
        fluxCCF,covCCF_line,nd_covCCF_line,contCCF=sub_new_compute_CCF(edge_mask_lines,wght_mask,n_RV,edge_wav,flux,cov,cal,resamp_mode)

    #Computing final covariance matrix
    #    - maximum dimension of covariance matrix from all contributing orders
    if cov is not None:
        nd_covCCF = np.amax(nd_covCCF_line) 
        covCCF = np.zeros([nd_covCCF,n_RV],dtype=float)
        for isub,wght_mask_line in enumerate(wght_mask):
            covCCF[0:nd_covCCF_line[isub],:] +=  covCCF_line[isub]         
    else:
        covCCF = None

    return fluxCCF,covCCF,contCCF


def sub_new_compute_CCF(edge_mask,wght_mask,n_RV,edge_wav,flux,cov,cal,resamp_mode):
    r"""**CCF computation** 
    
    Args:
        TDB
    
    Returns:
        None
    
    """   
    #Loop on selected lines
    #    - for each line in the line list, we co-add the contribution to the CCF
    #    - the bins have constant spectral width (which is the same for all lines in RV space, but changes with the line in wavelength space)
    nL_kept = len(wght_mask)
    fluxCCF = np.zeros(n_RV,dtype=float)
    contCCF = 0.
    covCCF_line = np.zeros(nL_kept,dtype=object)
    nd_covCCF_line = np.zeros(nL_kept,dtype=int)
    for isub,(edge_mask_line,wght_mask_line) in enumerate(zip(edge_mask,wght_mask)):

        #Spectrum around current line brought back from extracted to raw count units
        #    - if a calibration profile is provided as input we take its mean value in the local line range and use it to scale the input spectrum
        #      this is to get the spectrum as close as possible to its original count level, so that regions of the spectrum with comparable flux levels but different count levels do not contribute in the same way to the CCF
        #      the use of a constant estimated calibration rather than the actual profile is to keep the color balance of the spectrum intact, and avoid biasing the CCF
        if cal is not None:
            idxCCF_sub = np_where1D((edge_wav>=edge_mask_line[0]) & (edge_wav<=edge_mask_line[-1]))    #indexes where spectrum falls within mask line range
            idxCCF_sub_max = min([len(edge_wav)-1,idxCCF_sub[-1]])                                     
            mean_gainCCF_sub = np.mean(cal[idxCCF_sub[0]:idxCCF_sub_max+1])
        else:mean_gainCCF_sub =  1.

        #Spectrum around current line resampled on the CCF table
        if cov is None:
            fluxCCF_sub = bind.resampling(edge_mask_line,edge_wav, flux, kind=resamp_mode)/mean_gainCCF_sub                        
        else:
            fluxCCF_sub,covCCF_sub = bind.resampling(edge_mask_line,edge_wav, flux/mean_gainCCF_sub , cov = cov/mean_gainCCF_sub**2., kind=resamp_mode)   
            covCCF_line[isub]=(wght_mask_line**2.)*covCCF_sub
            nd_covCCF_line[isub] = np.shape(covCCF_line[isub])[0]                      

        #Add the weighted contribution of current line to the CCF
        fluxCCF += wght_mask_line*fluxCCF_sub
        contCCF += wght_mask_line/mean_gainCCF_sub

    return fluxCCF,covCCF_line,nd_covCCF_line,contCCF


def multithread_new_compute_CCF(func_input,nthreads,n_elem,y_inputs,common_args):
    r"""**Multithreading of sub_new_compute_CCF().**
    
    Specific implementation of `MAIN_multithread()`

    Args:
        func_input (function): multi-threaded function
        nthreads (int): number of threads
        n_elem (int): number of elements to thread
        y_inputs (list): threadable function inputs 
        common_args (tuple): common function inputs
    
    Returns:
        TBD
    
    """ 
    pool_proc = Pool(processes=nthreads)  
      
    #Indexes of chunks to be processed by each core       
    ind_chunk_list=init_parallel_func(nthreads,n_elem)

    #2 arrays with dimensions n_lines x n and n_lines
    chunked_args=[(y_inputs[0][ind_chunk[0]:ind_chunk[1],:],y_inputs[1][ind_chunk[0]:ind_chunk[1]])+common_args for ind_chunk in ind_chunk_list]				
         
    #------------------------------------------------------------------------------------	     					
    #Return the results from all cores as elements of a tuple
    #    - arguments could be given whole to map(), and be automatically divided using an input 'chunksize', however for long arrays it takes 
    # less time to do the chunking before
    all_results=tuple(tab for tab in pool_proc.starmap(func_input,chunked_args))

    #------------------------------------------------------------------------------------	 			
    #Outputs:
    #    - flux table co-added between processors	
    #    - covariance matrix and dimension table are appended between processors
    fluxCCF = np.sum(tuple(all_results[i][0] for i in range(nthreads)),axis=0)
    covCCF_line = np.concatenate(tuple(all_results[i][1] for i in range(nthreads)))
    nd_covCCF_line = np.concatenate(tuple(all_results[i][2] for i in range(nthreads)))
    contCCF = np.sum(tuple(all_results[i][3] for i in range(nthreads)))

    pool_proc.close()
    pool_proc.join()

    return fluxCCF,covCCF_line,nd_covCCF_line,contCCF










def conv_2D_to_1D_spec(data_type_gen,inst,vis,gen_dic,data_dic,prop_dic):
    """**Wrap-up function to compute 1D spectra.**
    
    Runs ANTARESS with default or manual settings.  
    
    Args:
        TBD
    
    Returns:
        None
    
    """ 
    data_vis=data_dic[inst][vis]
    gen_vis=gen_dic[inst][vis]
    dir_save = {} 
    iexp_conv,data_type_key,data_type = init_conversion(data_type_gen,gen_dic,prop_dic,inst,vis,'1Dfrom2D',dir_save,data_dic)

    #Paths
    proc_com_data_paths_new = gen_dic['save_data_dir']+'Processed_data/spec1D_'+inst+'_'+vis+'_com'
    
    #Calculating data
    if gen_dic['calc_spec_1D_'+data_type_gen]:
        print('         Calculating data')
        nthreads = gen_dic['nthreads_spec_1D_'+data_type_gen]
        if data_vis['type']!='spec2D':stop('Data must be 2D')
        nspec_1D = prop_dic['spec_1D_prop'][inst]['nspec']
        cen_bins_1D = prop_dic['spec_1D_prop'][inst]['cen_bins']
        edge_bins_1D = prop_dic['spec_1D_prop'][inst]['edge_bins']

        #Associated tables  
        proc_data_paths = {}
        scaling_data_paths = {} if gen_dic['flux_sc'] else None
        tell_data_paths =  {} if data_vis['tell_sp'] else None
        if gen_dic['cal_weight'] and data_vis['mean_gdet']:
            proc_weight=True
            mean_gdet_data_paths = {}
        else:
            proc_weight=False
            mean_gdet_data_paths = None
        DImast_weight_data_paths = {} if gen_dic['DImast_weight'] else None
        for key in data_type_key:   
            proc_data_paths[key] = data_vis['proc_'+key+'_data_paths']
            if scaling_data_paths is not None:scaling_data_paths[key] = data_vis['scaled_'+key+'_data_paths']
            if tell_data_paths is not None:tell_data_paths[key] = data_vis['tell_'+key+'_data_paths']
            if mean_gdet_data_paths is not None:mean_gdet_data_paths[key] = data_vis['mean_gdet_'+key+'_data_paths']
            if DImast_weight_data_paths is not None:DImast_weight_data_paths[key] = data_vis['mast_'+key+'_data_paths']
        LocEst_Atm_data_paths = data_vis['LocEst_Atm_data_paths'] if data_type_gen=='Atm' else None

        #Processing all exposures
        ifirst = iexp_conv[0]
        common_args = (data_type_gen,gen_dic['resamp_mode'],dir_save,cen_bins_1D,edge_bins_1D,nspec_1D,data_dic[inst]['nord'],ifirst,proc_com_data_paths_new,proc_weight,\
                       gen_dic[inst][vis]['idx_in2exp'],data_dic['Intr']['cov_loc_star'],proc_data_paths,tell_data_paths,scaling_data_paths,mean_gdet_data_paths,DImast_weight_data_paths,LocEst_Atm_data_paths,inst,vis,gen_dic['corr_Fbal'],gen_dic['corr_FbalOrd'],\
                       gen_dic['save_data_dir'],gen_dic['type'],data_vis['type'],data_vis['dim_exp'],gen_vis['idx_exp2in'],gen_vis['idx_in'])
        if nthreads>1: MAIN_multithread(conv_2D_to_1D_exp,nthreads,len(iexp_conv),[iexp_conv],common_args)                           
        else: conv_2D_to_1D_exp(iexp_conv,*common_args)  

        #Saving complementary data
        for gen in data_type_key: 
            rest_frame = data_dic[gen][inst][vis]['rest_frame']         #Rest frame propagated from original data       
            data_add = {'rest_frame':rest_frame,'dim_exp':[1,prop_dic['spec_1D_prop'][inst]['nspec']]}
            if gen=='Intr':data_add['iexp_conv'] = prop_dic[inst][vis]['idx_def'] 
            elif gen=='Res':data_add['iexp_conv'] = gen_dic[inst][vis]['idx_out']
            else:data_add['iexp_conv'] = iexp_conv
            datasave_npz(dir_save[gen]+'add',data_add)  

    else: 
        check_data({'path':proc_com_data_paths_new})
        
    #Updating paths
    #    - scaling is defined as a function and does not need updating
    data_vis['proc_com_data_paths'] = proc_com_data_paths_new
    for gen in dir_save:
        data_vis['proc_'+gen+'_data_paths'] = dir_save[gen]  
        if data_vis['tell_sp']:data_vis['tell_'+gen+'_data_paths'] = {}        
        data_vis['mast_'+gen+'_data_paths'] = {}    
    if data_type_gen=='Atm':data_vis['LocEst_Atm_data_paths'] = {}
    for iexp in iexp_conv:
        gen = deepcopy(data_type_gen)
        iexp_eff = deepcopy(iexp) 
        if data_type_gen=='Intr':         
            if (iexp in gen_vis['idx_in']):iexp_eff = gen_vis['idx_exp2in'][iexp] 
            else:gen = 'Res'
        data_vis['mast_'+gen+'_data_paths'][iexp_eff] = dir_save[gen]+'ref_'+str(iexp_eff)
        if data_vis['tell_sp']:data_vis['tell_'+data_type_gen+'_data_paths'][iexp] = dir_save[gen]+'_tell'+str(iexp_eff)
        if data_type_gen=='Atm':data_vis['LocEst_Atm_data_paths'][iexp] = dir_save[gen]+'estloc_'+str(iexp_eff)
      
    #Convert spectral mode 
    print('         ANTARESS switched to 1D processing')
    data_vis['mean_gdet'] = False 
    data_vis['comm_sp_tab']=True
    data_vis['type']='spec1D'
    data_vis['nspec'] = prop_dic['spec_1D_prop'][inst]['nspec']
    data_dic[inst]['nord'] = 1
    data_vis['dim_all'] = [data_vis['n_in_visit'],1,data_vis['nspec']]
    data_vis['dim_exp'] = [1,data_vis['nspec']]
    data_vis['dim_sp'] = [data_vis['n_in_visit'],1]

    #Continuum level and correction
    if data_type_gen=='Intr': 
        if data_dic['Intr']['calc_cont']:           
            data_dic['Intr'][inst][vis]['mean_cont']=calc_Intr_mean_cont(data_vis['n_in_tr'],data_dic[inst]['nord'],data_vis['nspec'],data_vis['proc_Intr_data_paths'],data_vis['type'],data_dic['Intr']['cont_range'],inst,data_dic['Intr']['cont_norm'],gen_dic['flag_err_inst'][inst])
            np.savez_compressed(data_vis['proc_Intr_data_paths']+'_add',data={'mean_cont':data_dic['Intr'][inst][vis]['mean_cont']},allow_pickle=True)
        else:
            check_data({'0':data_vis['proc_Intr_data_paths']+'_add'},silent=True)
            data_dic['Intr'][inst][vis]['mean_cont'] = dataload_npz(data_vis['proc_Intr_data_paths']+'_add')

    return None

def conv_2D_to_1D_exp(iexp_conv,data_type_gen,resamp_mode,dir_save,cen_bins_1D,edge_bins_1D,nspec_1D,nord,ifirst,proc_com_data_paths,proc_weight,\
                      idx_in2exp,cov_loc_star,proc_data_paths,tell_data_paths,scaling_data_paths,mean_gdet_data_paths,DImast_weight_data_paths,LocEst_Atm_data_paths,inst,vis,gen_corr_Fbal,gen_corr_Fbal_ord,\
                      save_data_dir,gen_type,data_mode,dim_exp,idx_exp2in,idx_in):
    r"""**Main routine to convert 2D spectra into 1D spectra** 
    
    Calculates 1D spectra from 2D spectra, propagating covariance matrix
     - 2D spectra are resampled over the common table and coadded
     - spectral values from different orders have to be equivalent at a given wavelength
     - conversion is applied to the latest processed data of each type
     - converted data is saved independently, but used as default data in all modules following the conversion

    Args:
        TDB
    
    Returns:
        None
    
    """ 
    #Processing each exposure
    #    - the conversion can be seen as the binning of all orders after they are resampled on a common table
    for iexp_sub,iexp in enumerate(iexp_conv):   
        data_type = deepcopy(data_type_gen)
        iexp_eff = deepcopy(iexp)     #Effective index (relative to global or in-transit tables)
        iexp_glob = deepcopy(iexp)    #Global index
        if data_type_gen=='DI':bdband_flux_sc = False
        else:
            bdband_flux_sc = True           
            if (data_type_gen=='Intr'):
                if (iexp in idx_in):
                    iexp_eff = idx_exp2in[iexp] 
                else:data_type = 'Res'
            elif data_type=='Absorption':iexp_glob = idx_in[iexp]
            
        #Upload spectra and associated tables in star or local frame
        flux_est_loc_exp = None
        cov_est_loc_exp = None
        SpSstar_spec = None    
        data_exp = dataload_npz(proc_data_paths[data_type]+str(iexp_eff))
        if tell_data_paths is not None:data_exp['tell'] = dataload_npz(tell_data_paths[data_type][iexp_eff])['tell'] 
        if proc_weight:data_exp['mean_gdet'] = dataload_npz(mean_gdet_data_paths[data_type][iexp_eff])['mean_gdet'] 
        else:data_exp['mean_gdet'] = None
        if DImast_weight_data_paths is not None:data_ref = dataload_npz(DImast_weight_data_paths[data_type][iexp_eff])
        if data_type_gen=='Atm':
            data_est_loc=dataload_npz(LocEst_Atm_data_paths[iexp_eff])
            flux_est_loc_exp = data_est_loc['flux']
            if cov_loc_star:cov_est_loc_exp = data_est_loc['cov']                      
            if data_type=='Absorption':SpSstar_spec = data_exp['SpSstar_spec']

        #Weight definition
        #    - cannot be parallelized as functions cannot be pickled
        #    - here the binning is performed between overlapping orders of the same exposure 
        #      all profiles that are the same for overlapping orders (tellurics, disk-integrated stellar spectrum, global flux scaling, ...) are thus not used in the weighing 
        #      they are however processed in the same way as the exposure if used later on in the pipeline 
        #    - for intrinsic and atmospheric profiles we provide the broadband flux scaling, even if does not matter to the weighing, because it is otherwise set to 0 and messes up with weights definition
        data_exp['weights'] = weights_bin_prof(range(nord),scaling_data_paths[data_type],inst,vis,gen_corr_Fbal,gen_corr_Fbal_ord,save_data_dir,gen_type,nord,iexp_glob,data_type,data_mode,dim_exp,None,data_exp['mean_gdet'], data_exp['cen_bins'],1.,np.ones(dim_exp),np.zeros([dim_exp[0],1,dim_exp[1]]),corr_Fbal=False,bdband_flux_sc=bdband_flux_sc)

        #Resample spectra and weights on 1D table in each order, and clean weights
        flux_exp_all,cov_exp_all,cond_def_all,glob_weight_all,cond_def_binned = pre_calc_bin_prof(nord,[nspec_1D],range(nord),resamp_mode,None,data_exp,edge_bins_1D)

        #Processing each order
        flux_ord_contr=[]
        cov_ord_contr=[] 
        if tell_data_paths is not None:tell_ord_contr = np.zeros(nspec_1D, dtype=float)
        if DImast_weight_data_paths is not None:
            flux_ref_ord_contr=[]
            cov_ref_ord_contr=[] 
        if SpSstar_spec is not None:SpSstar_spec_ord_contr = np.zeros(nspec_1D, dtype=float)
        if flux_est_loc_exp is not None:
            flux_est_loc_ord_contr=[]
            if cov_est_loc_exp is not None:cov_est_loc_ord_contr=[]            
        for iord in range(nord):

            #Multiply by order weight and store
            flux_ord,cov_ord = bind.mul_array(flux_exp_all[iord] , cov_exp_all[iord] , glob_weight_all[iord])          
            flux_ord_contr+=[flux_ord]
            cov_ord_contr+=[cov_ord]
            cond_def = cond_def_all[iord]

            #Apply same steps to complementary spectra
            #    - calibration profiles are not consistent in the overlaps between orders, and are not used anymore
            #    - spectral scaling is not updated, since it is defined as a function and remains applicable to the 1D spectrum as with the 2D spectrum 
            #    - the master has followed the same shifts as the intrinsic or atmospheric profiles, but always remain either defined on the common table, or on a specific table different from the table of its associated exposure
            if tell_data_paths is not None:
                tell_temp = bind.resampling(edge_bins_1D,data_exp['edge_bins'][iord],  data_exp['tell'][iord] ,kind=resamp_mode)
                tell_temp[~cond_def] = 0.                    
                tell_ord_contr+=tell_temp*glob_weight_all[iord]                     
            if DImast_weight_data_paths is not None:
                flux_ref_temp,cov_ref_temp = bind.resampling(edge_bins_1D, data_ref['edge_bins'][iord], data_ref['flux'][iord] , cov = data_ref['cov'][iord] , kind=resamp_mode)  
                flux_ref_temp,cov_ref_temp = bind.mul_array(flux_ref_temp,cov_ref_temp ,glob_weight_all[iord]      )
                flux_ref_temp[~cond_def] = 0.   
                flux_ref_ord_contr+=[flux_ref_temp]
                cov_ref_ord_contr+=[cov_ref_temp]
            if SpSstar_spec is not None:
                SpSstar_spec_temp = bind.resampling(edge_bins_1D,data_exp['edge_bins'][iord],SpSstar_spec[iord](data_exp['cen_bins'][iord]) ,kind=resamp_mode)
                SpSstar_spec_temp[~cond_def] = 0.
                SpSstar_spec_ord_contr+=SpSstar_spec_temp*glob_weight_all[iord]
            if flux_est_loc_exp is not None:  
                if cov_est_loc_exp is None:                    
                    flux_est_loc_temp = bind.resampling(edge_bins_1D, data_ref['edge_bins'][iord], flux_est_loc_exp[iord] , kind=resamp_mode)   
                    flux_est_loc_temp[~cond_def] = 0.   
                    flux_est_loc_ord_contr+=flux_est_loc_temp*glob_weight_all[iord]                  
                else:
                    flux_est_loc_temp,cov_est_loc_temp = bind.resampling(edge_bins_1D, data_ref['edge_bins'][iord], flux_est_loc_exp[iord] , cov = cov_est_loc_exp[iord], kind=resamp_mode)   
                    flux_est_loc_temp,cov_est_loc_temp = bind.mul_array( flux_est_loc_temp, cov_est_loc_temp , glob_weight_all[iord])
                    flux_est_loc_temp[~cond_def] = 0.   
                    flux_est_loc_ord_contr+=[flux_est_loc_temp]
                    cov_est_loc_ord_contr+=[cov_est_loc_temp]

        #Co-addition of spectra from all orders
        flux_1D,cov_1D = bind.sum(flux_ord_contr,cov_ord_contr)

        #Reset undefined pixels to nan
        flux_1D[~cond_def_binned]=np.nan        

        #Store data with artifical order for consistency with the routines
        #    - calibration profile are not used anymore afterward as there is no clear conversion from 2D to 1D for them
        #    - global flux scaling is not modified
        #      flux scaling tables are always called with global indexes
        data_exp1D = {'cen_bins':cen_bins_1D[None,:],'edge_bins':edge_bins_1D[None,:],'flux' : flux_1D[None,:],'cond_def' : cond_def_binned[None,:], 'cov' : [cov_1D]} 
        if SpSstar_spec is not None:data_exp1D['SpSstar_spec'] =  SpSstar_spec_ord_contr[None,:]
        datasave_npz(dir_save[data_type]+str(iexp_eff),data_exp1D)
        if tell_data_paths is not None:
            tell_1D = tell_ord_contr[None,:]
            datasave_npz(dir_save[data_type]+'_tell'+str(iexp_eff), {'tell':tell_1D})                 
        if DImast_weight_data_paths is not None:
            flux_ref_1D,cov_ref_1D = bind.sum(flux_ref_ord_contr,cov_ref_ord_contr)
            flux_ref_1D[~cond_def_binned]=np.nan   
            datasave_npz(dir_save[data_type]+'ref_'+str(iexp_eff),{'cen_bins':data_exp1D['cen_bins'],'edge_bins':data_exp1D['edge_bins'],'flux':flux_ref_1D[None,:],'cov':[cov_1D]})           
        if flux_est_loc_exp is not None:
            if cov_est_loc_exp is None:
                flux_est_loc_1D = flux_est_loc_ord_contr
                flux_est_loc_1D[~cond_def_binned]=np.nan 
                dic_sav_estloc = {'edge_bins':data_exp1D['edge_bins'],'flux':flux_est_loc_1D[None,:]}
            else:
                flux_est_loc_1D,cov_est_loc_1D = bind.sum(flux_est_loc_ord_contr,cov_est_loc_ord_contr)
                flux_est_loc_1D[~cond_def_binned]=np.nan   
                dic_sav_estloc = {'edge_bins':data_exp1D['edge_bins'],'flux':flux_est_loc_1D[None,:],'cov':[cov_est_loc_1D]}
            datasave_npz(dir_save[data_type]+'estloc_'+str(iexp_eff),dic_sav_estloc)

        #Update common table for the visit
        #    - set to the table of first processed exposure
        if iexp==ifirst:datasave_npz(proc_com_data_paths, {'dim_exp':[1,nspec_1D],'nspec':nspec_1D,'cen_bins':np.tile(cen_bins_1D,[1,1]),'edge_bins':np.tile(edge_bins_1D,[1,1])})      
    
    return None


